@@ -1,9 +1,9 @@
 /*
- * "$Id: lpd.c,v 1.56 2003/11/19 18:58:04 mike Exp $"
+ * "$Id: lpd.c,v 1.64 2004/06/29 02:44:47 mike Exp $"
  *
  *   Line Printer Daemon backend for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636-3142 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -71,8 +71,17 @@ static char	tmpfilename[1024] = "";	/* Temporary spool file name */
  * The order for control and data files in LPD requests...
  */
 
-#define ORDER_CONTROL_DATA	0
-#define ORDER_DATA_CONTROL	1
+#define ORDER_CONTROL_DATA	0	/* Control file first, then data */
+#define ORDER_DATA_CONTROL	1	/* Data file first, then control */
+
+
+/*
+ * What to reserve...
+ */
+
+#define RESERVE_NONE		0	/* Don't reserve a priviledged port */
+#define RESERVE_RFC1179		1	/* Reserve port 721-731 */
+#define RESERVE_ANY		2	/* Reserve port 1-1023 */
 
 
 /*
@@ -111,7 +120,7 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
 {
   char			method[255],	/* Method in URI */
 			hostname[1024],	/* Hostname */
-			username[255],	/* Username info (not used) */
+			username[255],	/* Username info */
 			resource[1024],	/* Resource info (printer name) */
 			*options,	/* Pointer to options */
 			name[255],	/* Name of option */
@@ -221,6 +230,15 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
 
   httpSeparate(argv[0], method, username, hostname, &port, resource);
 
+  if (!username[0])
+  {
+   /*
+    * If no username is in the device URI, then use the print job user...
+    */
+
+    strlcpy(username, argv[2], sizeof(username));
+  }
+
  /*
   * See if there are any options...
   */
@@ -228,11 +246,16 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
   banner         = 0;
   format         = 'l';
   order          = ORDER_CONTROL_DATA;
-  reserve        = 0;
+  reserve        = RESERVE_ANY;
   manual_copies  = 1;
   timeout        = 300;
   sanitize_title = 1;
 
+#if defined(__APPLE__)
+  /* We want to pass utf-8 characters, not re-map them (3071945) */
+  sanitize_title= 0;
+#endif
+
   if ((options = strchr(resource, '?')) != NULL)
   {
    /*
@@ -253,7 +276,8 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
       */
 
       for (ptr = name; *options && *options != '=';)
-        *ptr++ = *options++;
+        if (ptr < (name + sizeof(name) - 1))
+          *ptr++ = *options++;
       *ptr = '\0';
 
       if (*options == '=')
@@ -265,7 +289,8 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
         options ++;
 
 	for (ptr = value; *options && *options != '+';)
-          *ptr++ = *options++;
+          if (ptr < (value + sizeof(value) - 1))
+            *ptr++ = *options++;
 	*ptr = '\0';
 
 	if (*options == '+')
@@ -319,10 +344,16 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
         * Set port reservation mode...
 	*/
 
-        reserve = !value[0] ||
-	          strcasecmp(value, "on") == 0 ||
-	 	  strcasecmp(value, "yes") == 0 ||
-	 	  strcasecmp(value, "true") == 0;
+        if (!value[0] ||
+	    !strcasecmp(value, "on") ||
+	    !strcasecmp(value, "yes") ||
+	    !strcasecmp(value, "true") ||
+	    !strcasecmp(value, "rfc1179"))
+	  reserve = RESERVE_RFC1179;
+	else if (!strcasecmp(value, "any"))
+	  reserve = RESERVE_ANY;
+	else
+	  reserve = RESERVE_NONE;
       }
       else if (strcasecmp(name, "manual_copies") == 0)
       {
@@ -372,7 +403,7 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
     */
 
     for (ptr = title; *ptr; ptr ++)
-      if (!isalnum(*ptr) && !isspace(*ptr))
+      if (!isalnum(*ptr & 255) && !isspace(*ptr & 255))
 	*ptr = '_';
   }
 
@@ -394,7 +425,7 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
     }
 
     status = lpd_queue(hostname, port, resource + 1, filename,
-                       argv[2] /* user */, title, copies,
+                       username, title, copies,
 		       banner, format, order, reserve, manual_copies, timeout);
 
     if (!status)
@@ -402,7 +433,7 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
   }
   else
     status = lpd_queue(hostname, port, resource + 1, filename,
-                       argv[2] /* user */, title, 1,
+                       username, title, 1,
 		       banner, format, order, reserve, 1, timeout);
 
  /*
@@ -561,9 +592,24 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
     addr.sin_family = hostaddr->h_addrtype;
     addr.sin_port   = htons(port);
 
-    for (lport = 732;;)
+    for (lport = reserve == RESERVE_RFC1179 ? 732 : 1024;;)
     {
+     /*
+      * Choose the next priviledged port...
+      */
+
+      lport --;
+
+      if (lport < 721 && reserve == RESERVE_RFC1179)
+	lport = 731;
+      else if (lport < 1)
+	lport = 1023;
+
+#ifdef HAVE_GETEUID
+      if (geteuid() || !reserve)
+#else
       if (getuid() || !reserve)
+#endif /* HAVE_GETEUID */
       {
        /*
 	* Just create a regular socket...
@@ -581,13 +627,14 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
       {
        /*
 	* We're running as root and want to comply with RFC 1179.  Reserve a
-	* priviledged lport between 721 and 732...
+	* priviledged lport between 721 and 731...
 	*/
 
 	if ((fd = rresvport(&lport)) < 0)
 	{
 	  perror("ERROR: Unable to reserve port");
-	  sleep(30);
+	  sleep(1);
+
 	  continue;
 	}
       }
@@ -607,13 +654,15 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
 	}
 	else if (error == EADDRINUSE)
 	{
-	  port --;
-	  if (port < 721)
-	    port = 732;
+	 /*
+	  * Try on another port...
+	  */
+
+	  sleep(1);
 	}
 	else
 	{
-	  perror("ERROR: Unable to connect to printer");
+	  perror("ERROR: Unable to connect to printer; will retry in 30 seconds...");
           sleep(30);
 	}
       }
@@ -668,21 +717,21 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
 
     while (copies > 0)
     {
-      snprintf(cptr, sizeof(control) - (cptr - control), "%cdfA%03d%s\n", format,
+      snprintf(cptr, sizeof(control) - (cptr - control), "%cdfA%03d%.15s\n", format,
                getpid() % 1000, localhost);
       cptr   += strlen(cptr);
       copies --;
     }
 
     snprintf(cptr, sizeof(control) - (cptr - control),
-             "UdfA%03d%s\nN%s\n",
+             "UdfA%03d%.15s\nN%s\n",
              getpid() % 1000, localhost, title);
 
     fprintf(stderr, "DEBUG: Control file is:\n%s", control);
 
     if (order == ORDER_CONTROL_DATA)
     {
-      if (lpd_command(fd, timeout, "\002%d cfA%03.3d%s\n", strlen(control),
+      if (lpd_command(fd, timeout, "\002%d cfA%03.3d%.15s\n", strlen(control),
                       getpid() % 1000, localhost))
         return (1);
 
@@ -723,7 +772,7 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
       * Send the print file...
       */
 
-      if (lpd_command(fd, timeout, "\003%u dfA%03.3d%s\n",
+      if (lpd_command(fd, timeout, "\003%u dfA%03.3d%.15s\n",
                       (unsigned)filestats.st_size, getpid() % 1000,
 		      localhost))
         return (1);
@@ -788,7 +837,7 @@ lpd_queue(const char *hostname,		/* I - Host to connect to */
 
     if (status == 0 && order == ORDER_DATA_CONTROL)
     {
-      if (lpd_command(fd, timeout, "\002%d cfA%03.3d%s\n", strlen(control),
+      if (lpd_command(fd, timeout, "\002%d cfA%03.3d%.15s\n", strlen(control),
                       getpid() % 1000, localhost))
         return (1);
 
@@ -994,5 +1043,5 @@ sigterm_handler(int sig)		/* I - Signal */
 
 
 /*
- * End of "$Id: lpd.c,v 1.56 2003/11/19 18:58:04 mike Exp $".
+ * End of "$Id: lpd.c,v 1.64 2004/06/29 02:44:47 mike Exp $".
  */
