@@ -1,9 +1,9 @@
 /*
- * "$Id: scsi.c,v 1.6 2002/12/17 18:56:34 swdev Exp $"
+ * "$Id: scsi.c,v 1.7 2004/05/13 15:13:42 mike Exp $"
  *
  *   SCSI printer backend for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 2003 by Easy Software Products, all rights reserved.
+ *   Copyright 2003-2004 by Easy Software Products, all rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or
  *   without modification, are permitted provided that the
@@ -211,13 +211,10 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   if (fp != 0)
     close(fp);
 
-  if (!status)
-    fputs("INFO: Ready to print.\n", stderr);
-
   return (status);
 }
 
 
 /*
- * End of "$Id: scsi.c,v 1.6 2002/12/17 18:56:34 swdev Exp $".
+ * End of "$Id: scsi.c,v 1.7 2004/05/13 15:13:42 mike Exp $".
  */
