@@ -1,9 +1,9 @@
 /*
- * "$Id: serial.c,v 1.52 2003/09/17 19:26:18 mike Exp $"
+ * "$Id: serial.c,v 1.55 2004/05/13 15:13:42 mike Exp $"
  *
  *   Serial port backend for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636-3142 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -251,7 +251,8 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
       */
 
       for (ptr = name; *options && *options != '=';)
-        *ptr++ = *options++;
+        if (ptr < (name + sizeof(name) - 1))
+          *ptr++ = *options++;
       *ptr = '\0';
 
       if (*options == '=')
@@ -263,7 +264,8 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
         options ++;
 
 	for (ptr = value; *options && *options != '+';)
-          *ptr++ = *options++;
+          if (ptr < (value + sizeof(value) - 1))
+            *ptr++ = *options++;
 	*ptr = '\0';
 
 	if (*options == '+')
@@ -442,6 +444,8 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   if (bufsize > sizeof(buffer))
     bufsize = sizeof(buffer);
 
+  wbytes = 0;
+
   while (copies > 0)
   {
     copies --;
@@ -527,9 +531,7 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   if (fp != 0)
     close(fp);
 
-  fputs("INFO: Ready to print.\n", stderr);
-
-  return (0);
+  return (wbytes < 0);
 }
 
 
@@ -990,5 +992,5 @@ list_devices(void)
 
 
 /*
- * End of "$Id: serial.c,v 1.52 2003/09/17 19:26:18 mike Exp $".
+ * End of "$Id: serial.c,v 1.55 2004/05/13 15:13:42 mike Exp $".
  */
