@@ -1,9 +1,9 @@
 /*
- * "$Id: socket.c,v 1.39 2003/10/09 19:13:28 mike Exp $"
+ * "$Id: socket.c,v 1.42 2004/05/13 15:13:42 mike Exp $"
  *
  *   AppSocket backend for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636-3142 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -187,6 +187,8 @@ main(int  argc,			/* I - Number of command-line arguments (6 or 7) */
   addr.sin_family = hostaddr->h_addrtype;
   addr.sin_port   = htons(port);
 
+  wbytes = 0;
+
   while (copies > 0)
   {
     for (delay = 5;;)
@@ -369,9 +371,7 @@ main(int  argc,			/* I - Number of command-line arguments (6 or 7) */
   if (fp != 0)
     close(fp);
 
-  fputs("INFO: Ready to print.\n", stderr);
-
-  return (0);
+  return (wbytes < 0);
 }
 
 
@@ -414,5 +414,5 @@ print_backchannel(const unsigned char *buffer,	/* I - Data buffer */
 
 
 /*
- * End of "$Id: socket.c,v 1.39 2003/10/09 19:13:28 mike Exp $".
+ * End of "$Id: socket.c,v 1.42 2004/05/13 15:13:42 mike Exp $".
  */
