@@ -1,9 +1,9 @@
 /*
- * "$Id: lpc.c,v 1.16 2003/07/20 12:42:31 mike Exp $"
+ * "$Id: lpc.c,v 1.19 2004/02/25 20:14:50 mike Exp $"
  *
  *   "lpc" command for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products.
+ *   Copyright 1997-2004 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -99,18 +99,18 @@ main(int  argc,		/* I - Number of command-line arguments */
       * Find any options in the string...
       */
 
-      while (isspace(line[0]))
+      while (isspace(line[0] & 255))
         cups_strcpy(line, line + 1);
 
       for (params = line; *params != '\0'; params ++)
-        if (isspace(*params))
+        if (isspace(*params & 255))
 	  break;
 
      /*
       * Remove whitespace between the command and parameters...
       */
 
-      while (isspace(*params))
+      while (isspace(*params & 255))
         *params++ = '\0';
 
      /*
@@ -221,7 +221,8 @@ show_status(http_t *http,	/* I - HTTP connection to server */
 		*jattr;		/* Current job attribute */
   cups_lang_t	*language;	/* Default language */
   char		*printer,	/* Printer name */
-		*device;	/* Device URI */
+		*device,	/* Device URI */
+                *delimiter;     /* Char search result */
   ipp_pstate_t	pstate;		/* Printer state */
   int		accepting;	/* Is printer accepting jobs? */
   int		jobcount;	/* Count of current jobs */
@@ -351,7 +352,7 @@ show_status(http_t *http,	/* I - HTTP connection to server */
 	  * Skip leading whitespace and commas...
 	  */
 
-	  while (isspace(*dptr) || *dptr == ',')
+	  while (isspace(*dptr & 255) || *dptr == ',')
 	    dptr ++;
 
 	  if (*dptr == '\0')
@@ -365,7 +366,7 @@ show_status(http_t *http,	/* I - HTTP connection to server */
 	       *ptr != '\0' && *dptr != '\0' && *ptr == *dptr;
 	       ptr ++, dptr ++);
 
-          if (*ptr == '\0' && (*dptr == '\0' || *dptr == ',' || isspace(*dptr)))
+          if (*ptr == '\0' && (*dptr == '\0' || *dptr == ',' || isspace(*dptr & 255)))
 	  {
 	    match = 1;
 	    break;
@@ -375,9 +376,9 @@ show_status(http_t *http,	/* I - HTTP connection to server */
 	  * Skip trailing junk...
 	  */
 
-          while (!isspace(*dptr) && *dptr != '\0')
+          while (!isspace(*dptr & 255) && *dptr != '\0')
 	    dptr ++;
-	  while (isspace(*dptr) || *dptr == ',')
+	  while (isspace(*dptr & 255) || *dptr == ',')
 	    dptr ++;
 
 	  if (*dptr == '\0')
@@ -454,8 +455,11 @@ show_status(http_t *http,	/* I - HTTP connection to server */
 	  * Just show the method...
 	  */
 
-	  *strchr(device, ':') = '\0';
-	  printf("\tprinter is on device \'%s\' speed -1\n", device);
+	  if ((delimiter = strchr(device, ':')) != NULL )
+	  {
+	      *delimiter = '\0';
+	      printf("\tprinter is on device \'%s\' speed -1\n", device);
+	  }
 	}
 
 	printf("\tqueuing is %sabled\n", accepting ? "en" : "dis");
@@ -478,5 +482,5 @@ show_status(http_t *http,	/* I - HTTP connection to server */
 
 
 /*
- * End of "$Id: lpc.c,v 1.16 2003/07/20 12:42:31 mike Exp $".
+ * End of "$Id: lpc.c,v 1.19 2004/02/25 20:14:50 mike Exp $".
  */
