@@ -1,9 +1,9 @@
 /*
- * "$Id: lpq.c,v 1.26 2003/05/22 12:48:13 mike Exp $"
+ * "$Id: lpq.c,v 1.31 2004/09/08 19:01:34 mike Exp $"
  *
  *   "lpq" command for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products.
+ *   Copyright 1997-2004 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -66,9 +66,11 @@ main(int  argc,		/* I - Number of command-line arguments */
   int		i;		/* Looping var */
   http_t	*http;		/* Connection to server */
   const char	*dest,		/* Desired printer */
-		*user;		/* Desired user */
+		*user,		/* Desired user */
+		*val;		/* Environment variable name */
   char		*instance;	/* Printer instance */
   int		id,		/* Desired job ID */
+		all,		/* All printers */
 		interval,	/* Reporting interval */
 		longstatus;	/* Show file details */
   int		num_dests;	/* Number of destinations */
@@ -98,13 +100,10 @@ main(int  argc,		/* I - Number of command-line arguments */
   id         = 0;
   interval   = 0;
   longstatus = 0;
+  all        = 0;
 
   num_dests = cupsGetDests(&dests);
 
-  for (i = 0; i < num_dests; i ++)
-    if (dests[i].is_default)
-      dest = dests[i].name;
-
   for (i = 1; i < argc; i ++)
     if (argv[i][0] == '+')
       interval = atoi(argv[i] + 1);
@@ -158,7 +157,7 @@ main(int  argc,		/* I - Number of command-line arguments */
 	    break;
 
 	case 'a' : /* All printers */
-	    dest = NULL;
+	    all = 1;
 	    break;
 
 	case 'l' : /* Long status */
@@ -173,11 +172,45 @@ main(int  argc,		/* I - Number of command-line arguments */
 	    break;
       }
     }
-    else if (isdigit(argv[i][0]))
+    else if (isdigit(argv[i][0] & 255))
       id = atoi(argv[i]);
     else
       user = argv[i];
 
+  if (dest == NULL && !all)
+  {
+    for (i = 0; i < num_dests; i ++)
+      if (dests[i].is_default)
+	dest = dests[i].name;
+
+    if (dest == NULL)
+    {
+      val = NULL;
+
+      if ((dest = getenv("LPDEST")) == NULL)
+      {
+	if ((dest = getenv("PRINTER")) != NULL)
+	{
+          if (!strcmp(dest, "lp"))
+            dest = NULL;
+	  else
+	    val = "PRINTER";
+	}
+      }
+      else
+	val = "LPDEST";
+
+      if (dest && !cupsGetDest(dest, NULL, num_dests, dests))
+	fprintf(stderr, "lp: error - %s environment variable names non-existent destination \"%s\"!\n",
+        	val, dest);
+      else
+	fputs("lpq: error - no default destination available.\n", stderr);
+      httpClose(http);
+      cupsFreeDests(num_dests, dests);
+      return (1);
+    }
+  }
+
  /*
   * Show the status in a loop...
   */
@@ -425,7 +458,16 @@ show_jobs(http_t     *http,	/* I - HTTP connection to server */
 	strcpy(rankstr, "active");
       else
       {
-	snprintf(rankstr, sizeof(rankstr), "%d%s", rank, ranks[rank % 10]);
+       /*
+        * Make the rank show the "correct" suffix for each number
+	* (11-13 are the only special cases, for English anyways...)
+	*/
+
+	if ((rank % 100) >= 11 && (rank % 100) <= 13)
+	  snprintf(rankstr, sizeof(rankstr), "%dth", rank);
+	else
+	  snprintf(rankstr, sizeof(rankstr), "%d%s", rank, ranks[rank % 10]);
+
 	rank ++;
       }
 
@@ -569,5 +611,5 @@ usage(void)
 
 
 /*
- * End of "$Id: lpq.c,v 1.26 2003/05/22 12:48:13 mike Exp $".
+ * End of "$Id: lpq.c,v 1.31 2004/09/08 19:01:34 mike Exp $".
  */
