@@ -1,9 +1,9 @@
 /*
- * "$Id: classes.c,v 1.26 2003/11/07 18:52:38 mike Exp $"
+ * "$Id: classes.c,v 1.28 2004/02/25 21:56:58 mike Exp $"
  *
  *   Class status CGI for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products.
+ *   Copyright 1997-2004 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -133,7 +133,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 
         cgiSetVariable("DEFAULT_URI",
 	               ippRewriteURL(attr->values[0].string.text,
-		                     url, sizeof(url)));
+		                     url, sizeof(url), NULL));
       }
 
       ippDelete(response);
@@ -342,5 +342,5 @@ main(int  argc,				/* I - Number of command-line arguments */
 
 
 /*
- * End of "$Id: classes.c,v 1.26 2003/11/07 18:52:38 mike Exp $".
+ * End of "$Id: classes.c,v 1.28 2004/02/25 21:56:58 mike Exp $".
  */
