@@ -1,10 +1,10 @@
 /*
- * "$Id: dest.c,v 1.34 2003/10/30 15:45:20 mike Exp $"
+ * "$Id: dest.c,v 1.39 2004/06/29 03:44:52 mike Exp $"
  *
  *   User-defined destination (and option) support for the Common UNIX
  *   Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products.
+ *   Copyright 1997-2004 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -16,9 +16,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636-3142 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -30,6 +30,7 @@
  *   cupsFreeDests()   - Free the memory used by the list of destinations.
  *   cupsGetDest()     - Get the named destination from the list.
  *   cupsGetDests()    - Get the list of destinations.
+ *   cupsGetDests2()   - Get the list of destinations using a HTTP connection.
  *   cupsSetDests()    - Set the list of destinations.
  *   cups_get_dests()  - Get destinations from a file.
  *   cups_get_sdests() - Get destinations from a server.
@@ -52,7 +53,7 @@
 
 static int	cups_get_dests(const char *filename, int num_dests,
 		               cups_dest_t **dests);
-static int	cups_get_sdests(ipp_op_t op, int num_dests,
+static int	cups_get_sdests(http_t *http, ipp_op_t op, int num_dests,
 		                cups_dest_t **dests);
 
 
@@ -208,6 +209,33 @@ cupsGetDest(const char  *name,		/* I - Name of destination */
 
 int					/* O - Number of destinations */
 cupsGetDests(cups_dest_t **dests)	/* O - Destinations */
+{
+  int		num_dests;		/* Number of destinations */
+  http_t	*http;			/* HTTP connection */
+
+
+ /*
+  * Connect to the CUPS server and get the destination list and options...
+  */
+
+  http = httpConnectEncrypt(cupsServer(), ippPort(), cupsEncryption());
+
+  num_dests = cupsGetDests2(http, dests);
+
+  if (http)
+    httpClose(http);
+
+  return (num_dests);
+}
+
+
+/*
+ * 'cupsGetDests2()' - Get the list of destinations.
+ */
+
+int					/* O - Number of destinations */
+cupsGetDests2(http_t      *http,	/* I - HTTP connection */
+              cups_dest_t **dests)	/* O - Destinations */
 {
   int		i;			/* Looping var */
   int		num_dests;		/* Number of destinations */
@@ -221,6 +249,13 @@ cupsGetDests(cups_dest_t **dests)	/* O - Destinations */
   cups_dest_t	*reals;			/* Real queues */
 
 
+ /*
+  * Range check the input...
+  */
+
+  if (!http || !dests)
+    return (0);
+
  /*
   * Initialize destination array...
   */
@@ -232,8 +267,8 @@ cupsGetDests(cups_dest_t **dests)	/* O - Destinations */
   * Grab the printers and classes...
   */
 
-  num_dests = cups_get_sdests(CUPS_GET_PRINTERS, num_dests, dests);
-  num_dests = cups_get_sdests(CUPS_GET_CLASSES, num_dests, dests);
+  num_dests = cups_get_sdests(http, CUPS_GET_PRINTERS, num_dests, dests);
+  num_dests = cups_get_sdests(http, CUPS_GET_CLASSES, num_dests, dests);
 
  /*
   * Make a copy of the "real" queues for a later sanity check...
@@ -259,7 +294,7 @@ cupsGetDests(cups_dest_t **dests)	/* O - Destinations */
   * Grab the default destination...
   */
 
-  if ((defprinter = cupsGetDefault()) != NULL)
+  if ((defprinter = cupsGetDefault2(http)) != NULL)
   {
    /*
     * Grab printer and instance name...
@@ -362,6 +397,31 @@ cupsGetDests(cups_dest_t **dests)	/* O - Destinations */
 void
 cupsSetDests(int         num_dests,	/* I - Number of destinations */
              cups_dest_t *dests)	/* I - Destinations */
+{
+  http_t	*http;			/* HTTP connection */
+
+
+ /*
+  * Connect to the CUPS server and save the destination list and options...
+  */
+
+  http = httpConnectEncrypt(cupsServer(), ippPort(), cupsEncryption());
+
+  cupsSetDests2(http, num_dests, dests);
+
+  if (http)
+    httpClose(http);
+}
+
+
+/*
+ * 'cupsSetDests()' - Set the list of destinations.
+ */
+
+int					/* O - 0 on success, -1 on error */
+cupsSetDests2(http_t      *http,	/* I - HTTP connection */
+              int         num_dests,	/* I - Number of destinations */
+              cups_dest_t *dests)	/* I - Destinations */
 {
   int		i, j;			/* Looping vars */
   int		wrote;			/* Wrote definition? */
@@ -376,12 +436,19 @@ cupsSetDests(int         num_dests,	/* I - Number of destinations */
   const char	*val;			/* Value of temporary option */
 
 
+ /*
+  * Range check the input...
+  */
+
+  if (!http || !num_dests || !dests)
+    return (-1);
+
  /*
   * Get the server destinations...
   */
 
-  num_temps = cups_get_sdests(CUPS_GET_PRINTERS, 0, &temps);
-  num_temps = cups_get_sdests(CUPS_GET_CLASSES, num_temps, &temps);
+  num_temps = cups_get_sdests(http, CUPS_GET_PRINTERS, 0, &temps);
+  num_temps = cups_get_sdests(http, CUPS_GET_CLASSES, num_temps, &temps);
 
  /*
   * Figure out which file to write to...
@@ -417,7 +484,7 @@ cupsSetDests(int         num_dests,	/* I - Number of destinations */
   if ((fp = fopen(filename, "w")) == NULL)
   {
     cupsFreeDests(num_temps, temps);
-    return;
+    return (-1);
   }
 
  /*
@@ -495,7 +562,9 @@ cupsSetDests(int         num_dests,	/* I - Number of destinations */
   * Close the file and return...
   */
 
-  fclose(fp);      
+  fclose(fp);
+
+  return (0);
 }
 
 
@@ -547,9 +616,9 @@ cups_get_dests(const char  *filename,	/* I - File to read from */
     * See what type of line it is...
     */
 
-    if (strncasecmp(line, "dest", 4) == 0 && isspace(line[4]))
+    if (strncasecmp(line, "dest", 4) == 0 && isspace(line[4] & 255))
       lineptr = line + 4;
-    else if (strncasecmp(line, "default", 7) == 0 && isspace(line[7]))
+    else if (strncasecmp(line, "default", 7) == 0 && isspace(line[7] & 255))
       lineptr = line + 7;
     else
       continue;
@@ -558,7 +627,7 @@ cups_get_dests(const char  *filename,	/* I - File to read from */
     * Skip leading whitespace...
     */
 
-    while (isspace(*lineptr))
+    while (isspace(*lineptr & 255))
       lineptr ++;
 
     if (!*lineptr)
@@ -570,7 +639,7 @@ cups_get_dests(const char  *filename,	/* I - File to read from */
     * Search for an instance...
     */
 
-    while (!isspace(*lineptr) && *lineptr && *lineptr != '/')
+    while (!isspace(*lineptr & 255) && *lineptr && *lineptr != '/')
       lineptr ++;
 
     if (!*lineptr)
@@ -589,7 +658,7 @@ cups_get_dests(const char  *filename,	/* I - File to read from */
       * Search for an instance...
       */
 
-      while (!isspace(*lineptr) && *lineptr)
+      while (!isspace(*lineptr & 255) && *lineptr)
 	lineptr ++;
     }
     else
@@ -656,12 +725,12 @@ cups_get_dests(const char  *filename,	/* I - File to read from */
  */
 
 static int				/* O - Number of destinations */
-cups_get_sdests(ipp_op_t    op,		/* I - get-printers or get-classes */
+cups_get_sdests(http_t      *http,	/* I - HTTP connection */
+                ipp_op_t    op,		/* I - get-printers or get-classes */
                 int         num_dests,	/* I - Number of destinations */
                 cups_dest_t **dests)	/* IO - Destinations */
 {
   cups_dest_t	*dest;			/* Current destination */
-  http_t	*http;			/* HTTP connection */
   ipp_t		*request,		/* IPP Request */
 		*response;		/* IPP Response */
   ipp_attribute_t *attr;		/* Current attribute */
@@ -675,13 +744,6 @@ cups_get_sdests(ipp_op_t    op,		/* I - get-printers or get-classes */
 		};
 
 
- /*
-  * Connect to the CUPS server...
-  */
-
-  if ((http = httpConnect(cupsServer(), ippPort())) == NULL)
-    return (num_dests);
-
  /*
   * Build a CUPS_GET_PRINTERS or CUPS_GET_CLASSES request, which require
   * the following attributes:
@@ -781,12 +843,6 @@ cups_get_sdests(ipp_op_t    op,		/* I - get-printers or get-classes */
     ippDelete(response);
   }
 
- /*
-  * Close the server connection...
-  */
-
-  httpClose(http);
-
  /*
   * Return the count...
   */
@@ -796,5 +852,5 @@ cups_get_sdests(ipp_op_t    op,		/* I - get-printers or get-classes */
 
 
 /*
- * End of "$Id: dest.c,v 1.34 2003/10/30 15:45:20 mike Exp $".
+ * End of "$Id: dest.c,v 1.39 2004/06/29 03:44:52 mike Exp $".
  */
