@@ -1,9 +1,9 @@
 /*
- * "$Id: http-addr.c,v 1.3 2002/12/17 18:56:41 swdev Exp $"
+ * "$Id: http-addr.c,v 1.9 2004/06/29 03:44:52 mike Exp $"
  *
  *   HTTP host/address routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636-3142 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -55,16 +55,17 @@ httpGetHostByName(const char *name)	/* I - Hostname or IP address */
   static char		*packed_ptr[2];	/* Pointer to packed address */
   static struct hostent	host_ip;	/* Host entry for IP address */
 
+
 #if defined(__APPLE__)
   /* OS X hack to avoid it's ocassional long delay in lookupd */
-  static char sLoopback[] = "127.0.0.1";
+  static const char sLoopback[] = "127.0.0.1";
   if (strcmp(name, "localhost") == 0)
     name = sLoopback;
 #endif /* __APPLE__ */
 
  /*
   * This function is needed because some operating systems have a
-  * buggy implementation of httpGetHostByName() that does not support
+  * buggy implementation of gethostbyname() that does not support
   * IP addresses.  If the first character of the name string is a
   * number, then sscanf() is used to extract the IP components.
   * We then pack the components into an IPv4 address manually,
@@ -72,7 +73,7 @@ httpGetHostByName(const char *name)	/* I - Hostname or IP address */
   * htonl() macro to get the right byte order for the address.
   */
 
-  for (nameptr = name; isdigit(*nameptr) || *nameptr == '.'; nameptr ++);
+  for (nameptr = name; isdigit(*nameptr & 255) || *nameptr == '.'; nameptr ++);
 
   if (!*nameptr)
   {
@@ -83,7 +84,10 @@ httpGetHostByName(const char *name)	/* I - Hostname or IP address */
     */
 
     if (sscanf(name, "%u.%u.%u.%u", ip, ip + 1, ip + 2, ip + 3) != 4)
-      return (NULL); /* Must have 4 numbers */
+      return (NULL);			/* Must have 4 numbers */
+
+    if (ip[0] > 255 || ip[1] > 255 || ip[2] > 255 || ip[3] > 255)
+      return (NULL);			/* Invalid byte ranges! */
 
     packed_ip = htonl(((((((ip[0] << 8) | ip[1]) << 8) | ip[2]) << 8) | ip[3]));
 
@@ -114,5 +118,5 @@ httpGetHostByName(const char *name)	/* I - Hostname or IP address */
 
 
 /*
- * End of "$Id: http-addr.c,v 1.3 2002/12/17 18:56:41 swdev Exp $".
+ * End of "$Id: http-addr.c,v 1.9 2004/06/29 03:44:52 mike Exp $".
  */
