@@ -1,9 +1,9 @@
 /*
- * "$Id: http.c,v 1.127 2003/11/19 18:05:55 mike Exp $"
+ * "$Id: http.c,v 1.142 2004/08/18 15:36:34 mike Exp $"
  *
  *   HTTP routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636-3142 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -55,7 +55,9 @@
  *   httpGetDateTime()    - Get a time value from a formatted date/time string.
  *   httpUpdate()         - Update the current HTTP state for incoming data.
  *   httpDecode64()       - Base64-decode a string.
+ *   httpDecode64_2()     - Base64-decode a string.
  *   httpEncode64()       - Base64-encode a string.
+ *   httpEncode64_2()     - Base64-encode a string.
  *   httpGetLength()      - Get the amount of data remaining from the
  *                          content-length or transfer-encoding fields.
  *   http_field()         - Return the field index for a field name.
@@ -204,8 +206,8 @@ httpInitialize(void)
 #endif /* HAVE_LIBSSL */
 
 #ifdef WIN32
-  WSADATA	winsockdata;	/* WinSock data */
-  static int	initialized = 0;/* Has WinSock been initialized? */
+  WSADATA	winsockdata;		/* WinSock data */
+  static int	initialized = 0;	/* Has WinSock been initialized? */
 
 
   if (!initialized)
@@ -213,7 +215,7 @@ httpInitialize(void)
 #elif defined(HAVE_SIGSET)
   sigset(SIGPIPE, SIG_IGN);
 #elif defined(HAVE_SIGACTION)
-  struct sigaction	action;	/* POSIX sigaction data */
+  struct sigaction	action;		/* POSIX sigaction data */
 
 
  /*
@@ -290,6 +292,8 @@ httpClearCookie(http_t *http)			/* I - Connection */
 void
 httpClose(http_t *http)		/* I - Connection to close */
 {
+  DEBUG_printf(("httpClose(http=%p)\n", http));
+
   if (!http)
     return;
 
@@ -353,7 +357,10 @@ httpConnectEncrypt(const char *host,	/* I - Host to connect to */
   struct hostent	*hostaddr;	/* Host address data */
 
 
-  if (host == NULL)
+  DEBUG_printf(("httpConnectEncrypt(host=\"%s\", port=%d, encrypt=%d)\n",
+                host ? host : "(null)", port, encrypt));
+
+  if (!host)
     return (NULL);
 
   httpInitialize();
@@ -456,6 +463,8 @@ int					/* O - -1 on error, 0 on success */
 httpEncryption(http_t            *http,	/* I - HTTP data */
                http_encryption_t e)	/* I - New encryption preference */
 {
+  DEBUG_printf(("httpEncryption(http=%p, e=%d)\n", http, e));
+
 #ifdef HAVE_SSL
   if (!http)
     return (0);
@@ -487,6 +496,12 @@ httpReconnect(http_t *http)	/* I - HTTP data */
 {
   int		val;		/* Socket option value */
 
+
+  DEBUG_printf(("httpReconnect(http=%p)\n", http));
+
+  if (!http)
+    return (-1);
+
 #ifdef HAVE_SSL
   if (http->tls)
     http_shutdown_ssl(http);
@@ -614,22 +629,22 @@ httpGetSubField(http_t       *http,	/* I - HTTP data */
 		*ptr;			/* Pointer into string buffer */
 
 
+  DEBUG_printf(("httpGetSubField(http=%p, field=%d, name=\"%s\", value=%p)\n",
+                http, field, name, value));
+
   if (http == NULL ||
       field < HTTP_FIELD_ACCEPT_LANGUAGE ||
       field > HTTP_FIELD_WWW_AUTHENTICATE ||
       name == NULL || value == NULL)
     return (NULL);
 
-  DEBUG_printf(("httpGetSubField(%p, %d, \"%s\", %p)\n",
-                http, field, name, value));
-
   for (fptr = http->fields[field]; *fptr;)
   {
    /*
     * Skip leading whitespace...
     */
 
-    while (isspace(*fptr))
+    while (isspace(*fptr & 255))
       fptr ++;
 
     if (*fptr == ',')
@@ -643,18 +658,18 @@ httpGetSubField(http_t       *http,	/* I - HTTP data */
     */
 
     for (ptr = temp;
-         *fptr && *fptr != '=' && !isspace(*fptr) && ptr < (temp + sizeof(temp) - 1);
+         *fptr && *fptr != '=' && !isspace(*fptr & 255) && ptr < (temp + sizeof(temp) - 1);
          *ptr++ = *fptr++);
 
     *ptr = '\0';
 
-    DEBUG_printf(("name = \"%s\"\n", temp));
+    DEBUG_printf(("httpGetSubField: name=\"%s\"\n", temp));
 
    /*
     * Skip trailing chars up to the '='...
     */
 
-    while (isspace(*fptr))
+    while (isspace(*fptr & 255))
       fptr ++;
 
     if (!*fptr)
@@ -669,7 +684,7 @@ httpGetSubField(http_t       *http,	/* I - HTTP data */
 
     fptr ++;
 
-    while (isspace(*fptr))
+    while (isspace(*fptr & 255))
       fptr ++;
 
     if (*fptr == '\"')
@@ -697,16 +712,16 @@ httpGetSubField(http_t       *http,	/* I - HTTP data */
       */
 
       for (ptr = value;
-           *fptr && !isspace(*fptr) && *fptr != ',' && ptr < (value + HTTP_MAX_VALUE - 1);
+           *fptr && !isspace(*fptr & 255) && *fptr != ',' && ptr < (value + HTTP_MAX_VALUE - 1);
 	   *ptr++ = *fptr++);
 
       *ptr = '\0';
 
-      while (*fptr && !isspace(*fptr) && *fptr != ',')
+      while (*fptr && !isspace(*fptr & 255) && *fptr != ',')
         fptr ++;
     }
 
-    DEBUG_printf(("value = \"%s\"\n", value));
+    DEBUG_printf(("httpGetSubField: value=\"%s\"\n", value));
 
    /*
     * See if this is the one...
@@ -834,15 +849,14 @@ httpTrace(http_t     *http,		/* I - HTTP data */
  */
 
 void
-httpFlush(http_t *http)	/* I - HTTP data */
+httpFlush(http_t *http)			/* I - HTTP data */
 {
-  char	buffer[8192];	/* Junk buffer */
+  char	buffer[8192];			/* Junk buffer */
 
 
-  if (http->state != HTTP_WAITING)
-  {
-    while (httpRead(http, buffer, sizeof(buffer)) > 0);
-  }
+  DEBUG_printf(("httpFlush(http=%p), state=%d\n", http, http->state));
+
+  while (httpRead(http, buffer, sizeof(buffer)) > 0);
 }
 
 
@@ -859,7 +873,8 @@ httpRead(http_t *http,			/* I - HTTP data */
   char		len[32];		/* Length string */
 
 
-  DEBUG_printf(("httpRead(%p, %p, %d)\n", http, buffer, length));
+  DEBUG_printf(("httpRead(http=%p, buffer=%p, length=%d)\n",
+                http, buffer, length));
 
   if (http == NULL || buffer == NULL)
     return (-1);
@@ -888,7 +903,7 @@ httpRead(http_t *http,			/* I - HTTP data */
     }
   }
 
-  DEBUG_printf(("httpRead: data_remaining = %d\n", http->data_remaining));
+  DEBUG_printf(("httpRead: data_remaining=%d\n", http->data_remaining));
 
   if (http->data_remaining <= 0)
   {
@@ -905,6 +920,12 @@ httpRead(http_t *http,			/* I - HTTP data */
     else
       http->state = HTTP_WAITING;
 
+   /*
+    * Prevent future reads for this request...
+    */
+
+    http->data_encoding = HTTP_ENCODE_LENGTH;
+
     return (0);
   }
   else if (length > http->data_remaining)
@@ -1179,7 +1200,7 @@ httpWrite(http_t     *http,		/* I - HTTP data */
 #else
       if (errno == EINTR)
         continue;
-      else if (errno != http->error)
+      else if (errno != http->error && errno != ECONNRESET)
       {
         http->error = errno;
 	continue;
@@ -1269,7 +1290,7 @@ httpGets(char   *line,			/* I - Line to read into */
   int	bytes;				/* Number of bytes read */
 
 
-  DEBUG_printf(("httpGets(%p, %d, %p)\n", line, length, http));
+  DEBUG_printf(("httpGets(line=%p, length=%d, http=%p)\n", line, length, http));
 
   if (http == NULL || line == NULL)
     return (NULL);
@@ -1311,6 +1332,8 @@ httpGets(char   *line,			/* I - Line to read into */
 #endif /* HAVE_SSL */
         bytes = recv(http->fd, bufend, HTTP_MAX_BUFFER - http->used, 0);
 
+      DEBUG_printf(("httpGets: read %d bytes...\n", bytes));
+
       if (bytes < 0)
       {
        /*
@@ -1324,8 +1347,10 @@ httpGets(char   *line,			/* I - Line to read into */
 	  continue;
 	}
 
-        DEBUG_printf(("httpGets(): recv() error %d!\n", WSAGetLastError()));
+        DEBUG_printf(("httpGets: recv() error %d!\n", WSAGetLastError()));
 #else
+        DEBUG_printf(("httpGets: recv() error %d!\n", errno));
+
         if (errno == EINTR)
 	  continue;
 	else if (errno != http->error)
@@ -1333,8 +1358,6 @@ httpGets(char   *line,			/* I - Line to read into */
 	  http->error = errno;
 	  continue;
 	}
-
-        DEBUG_printf(("httpGets(): recv() error %d!\n", errno));
 #endif /* WIN32 */
 
         return (NULL);
@@ -1391,11 +1414,11 @@ httpGets(char   *line,			/* I - Line to read into */
     if (http->used > 0)
       memmove(http->buffer, bufptr, http->used);
 
-    DEBUG_printf(("httpGets(): Returning \"%s\"\n", line));
+    DEBUG_printf(("httpGets: Returning \"%s\"\n", line));
     return (line);
   }
 
-  DEBUG_puts("httpGets(): No new line available!");
+  DEBUG_puts("httpGets: No new line available!");
 
   return (NULL);
 }
@@ -1418,6 +1441,8 @@ httpPrintf(http_t     *http,		/* I - HTTP data */
   va_list	ap;			/* Variable argument pointer */
 
 
+  DEBUG_printf(("httpPrintf(http=%p, format=\"%s\", ...)\n", http, format));
+
   va_start(ap, format);
   bytes = vsnprintf(buf, sizeof(buf), format, ap);
   va_end(ap);
@@ -1527,11 +1552,11 @@ httpUpdate(http_t *http)		/* I - HTTP data */
   char		line[1024],		/* Line from connection... */
 		*value;			/* Pointer to value on line */
   http_field_t	field;			/* Field index */
-  int		major, minor;		/* HTTP version numbers */
-  http_status_t	status;			/* Authorization status */
+  int		major, minor,		/* HTTP version numbers */
+		status;			/* Request status */
 
 
-  DEBUG_printf(("httpUpdate(%p)\n", http));
+  DEBUG_printf(("httpUpdate(http=%p), state=%d\n", http, http->state));
 
  /*
   * If we haven't issued any commands, then there is nothing to "update"...
@@ -1546,7 +1571,7 @@ httpUpdate(http_t *http)		/* I - HTTP data */
 
   while (httpGets(line, sizeof(line), http) != NULL)
   {
-    DEBUG_puts(line);
+    DEBUG_printf(("httpUpdate: Got \"%s\"\n", line));
 
     if (line[0] == '\0')
     {
@@ -1562,16 +1587,19 @@ httpUpdate(http_t *http)		/* I - HTTP data */
       if (http->status == HTTP_CONTINUE)
         return (http->status);
 
+      if (http->status < HTTP_BAD_REQUEST)
+        http->digest_tries = 0;
+
 #ifdef HAVE_SSL
       if (http->status == HTTP_SWITCHING_PROTOCOLS && !http->tls)
       {
 	if (http_setup_ssl(http) != 0)
 	{
-#ifdef WIN32
+#  ifdef WIN32
 	  closesocket(http->fd);
-#else
+#  else
 	  close(http->fd);
-#endif
+#  endif /* WIN32 */
 
 	  return (HTTP_ERROR);
 	}
@@ -1605,11 +1633,11 @@ httpUpdate(http_t *http)		/* I - HTTP data */
       * Got the beginning of a response...
       */
 
-      if (sscanf(line, "HTTP/%d.%d%d", &major, &minor, (int *)&status) != 3)
+      if (sscanf(line, "HTTP/%d.%d%d", &major, &minor, &status) != 3)
         return (HTTP_ERROR);
 
       http->version = (http_version_t)(major * 100 + minor);
-      http->status  = status;
+      http->status  = (http_status_t)status;
     }
     else if ((value = strchr(line, ':')) != NULL)
     {
@@ -1618,7 +1646,7 @@ httpUpdate(http_t *http)		/* I - HTTP data */
       */
 
       *value++ = '\0';
-      while (isspace(*value))
+      while (isspace(*value & 255))
         value ++;
 
      /*
@@ -1660,8 +1688,13 @@ httpUpdate(http_t *http)		/* I - HTTP data */
   * See if there was an error...
   */
 
+  if (http->error == EPIPE && http->status > HTTP_CONTINUE)
+    return (http->status);
+
   if (http->error)
   {
+    DEBUG_printf(("httpUpdate: socket error %d - %s\n", http->error,
+                  strerror(http->error)));
     http->status = HTTP_ERROR;
     return (HTTP_ERROR);
   }
@@ -1678,16 +1711,50 @@ httpUpdate(http_t *http)		/* I - HTTP data */
  * 'httpDecode64()' - Base64-decode a string.
  */
 
-char *				/* O - Decoded string */
-httpDecode64(char       *out,	/* I - String to write to */
-             const char *in)	/* I - String to read from */
+char *					/* O - Decoded string */
+httpDecode64(char       *out,		/* I - String to write to */
+             const char *in)		/* I - String to read from */
 {
-  int	pos,			/* Bit position */
-	base64;			/* Value of this character */
-  char	*outptr;		/* Output pointer */
+  int	outlen;				/* Output buffer length */
 
 
-  for (outptr = out, pos = 0; *in != '\0'; in ++)
+ /*
+  * Use the old maximum buffer size for binary compatibility...
+  */
+
+  outlen = 512;
+
+  return (httpDecode64_2(out, &outlen, in));
+}
+
+
+/*
+ * 'httpDecode64_2()' - Base64-decode a string.
+ */
+
+char *					/* O  - Decoded string */
+httpDecode64_2(char       *out,		/* I  - String to write to */
+	       int        *outlen,	/* IO - Size of output string */
+               const char *in)		/* I  - String to read from */
+{
+  int	pos,				/* Bit position */
+	base64;				/* Value of this character */
+  char	*outptr,			/* Output pointer */
+	*outend;			/* End of output buffer */
+
+
+ /*
+  * Range check input...
+  */
+
+  if (!out || !outlen || *outlen < 1 || !in || !*in)
+    return (NULL);
+
+ /*
+  * Convert from base-64 to bytes...
+  */
+
+  for (outptr = out, outend = out + *outlen - 1, pos = 0; *in != '\0'; in ++)
   {
    /*
     * Decode this character into a number from 0 to 63...
@@ -1715,21 +1782,27 @@ httpDecode64(char       *out,	/* I - String to write to */
     switch (pos)
     {
       case 0 :
-          *outptr = base64 << 2;
+          if (outptr < outend)
+            *outptr = base64 << 2;
 	  pos ++;
 	  break;
       case 1 :
-          *outptr++ |= (base64 >> 4) & 3;
-	  *outptr = (base64 << 4) & 255;
+          if (outptr < outend)
+            *outptr++ |= (base64 >> 4) & 3;
+          if (outptr < outend)
+	    *outptr = (base64 << 4) & 255;
 	  pos ++;
 	  break;
       case 2 :
-          *outptr++ |= (base64 >> 2) & 15;
-	  *outptr = (base64 << 6) & 255;
+          if (outptr < outend)
+            *outptr++ |= (base64 >> 2) & 15;
+          if (outptr < outend)
+	    *outptr = (base64 << 6) & 255;
 	  pos ++;
 	  break;
       case 3 :
-          *outptr++ |= base64;
+          if (outptr < outend)
+            *outptr++ |= base64;
 	  pos = 0;
 	  break;
     }
@@ -1738,9 +1811,11 @@ httpDecode64(char       *out,	/* I - String to write to */
   *outptr = '\0';
 
  /*
-  * Return the decoded string...
+  * Return the decoded string and size...
   */
 
+  *outlen = (int)(outptr - out);
+
   return (out);
 }
 
@@ -1749,12 +1824,27 @@ httpDecode64(char       *out,	/* I - String to write to */
  * 'httpEncode64()' - Base64-encode a string.
  */
 
-char *				/* O - Encoded string */
-httpEncode64(char       *out,	/* I - String to write to */
-             const char *in)	/* I - String to read from */
+char *					/* O - Encoded string */
+httpEncode64(char       *out,		/* I - String to write to */
+             const char *in)		/* I - String to read from */
 {
-  char		*outptr;	/* Output pointer */
-  static const char base64[] =	/* Base64 characters... */
+  return (httpEncode64_2(out, 512, in, strlen(in)));
+}
+
+
+/*
+ * 'httpEncode64_2()' - Base64-encode a string.
+ */
+
+char *					/* O - Encoded string */
+httpEncode64_2(char       *out,		/* I - String to write to */
+	       int        outlen,	/* I - Size of output string */
+               const char *in,		/* I - String to read from */
+	       int        inlen)	/* I - Size of input string */
+{
+  char		*outptr,		/* Output pointer */
+		*outend;		/* End of output buffer */
+  static const char base64[] =		/* Base64 characters... */
   		{
 		  "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
 		  "abcdefghijklmnopqrstuvwxyz"
@@ -1763,33 +1853,53 @@ httpEncode64(char       *out,	/* I - String to write to */
   		};
 
 
-  for (outptr = out; *in != '\0'; in ++)
+ /*
+  * Range check input...
+  */
+
+  if (!out || outlen < 1 || !in || inlen < 1)
+    return (NULL);
+
+ /*
+  * Convert bytes to base-64...
+  */
+
+  for (outptr = out, outend = out + outlen - 1; inlen > 0; in ++, inlen --)
   {
    /*
     * Encode the up to 3 characters as 4 Base64 numbers...
     */
 
-    *outptr ++ = base64[in[0] >> 2];
-    *outptr ++ = base64[((in[0] << 4) | (in[1] >> 4)) & 63];
+    if (outptr < outend)
+      *outptr ++ = base64[(in[0] & 255) >> 2];
+    if (outptr < outend)
+      *outptr ++ = base64[(((in[0] & 255) << 4) | ((in[1] & 255) >> 4)) & 63];
 
     in ++;
-    if (*in == '\0')
+    inlen --;
+    if (inlen <= 0)
     {
-      *outptr ++ = '=';
-      *outptr ++ = '=';
+      if (outptr < outend)
+        *outptr ++ = '=';
+      if (outptr < outend)
+        *outptr ++ = '=';
       break;
     }
 
-    *outptr ++ = base64[((in[0] << 2) | (in[1] >> 6)) & 63];
+    if (outptr < outend)
+      *outptr ++ = base64[(((in[0] & 255) << 2) | ((in[1] & 255) >> 6)) & 63];
 
     in ++;
-    if (*in == '\0')
+    inlen --;
+    if (inlen <= 0)
     {
-      *outptr ++ = '=';
+      if (outptr < outend)
+        *outptr ++ = '=';
       break;
     }
 
-    *outptr ++ = base64[in[0] & 63];
+    if (outptr < outend)
+      *outptr ++ = base64[in[0] & 63];
   }
 
   *outptr = '\0';
@@ -1810,7 +1920,7 @@ httpEncode64(char       *out,	/* I - String to write to */
 int				/* O - Content length */
 httpGetLength(http_t *http)	/* I - HTTP data */
 {
-  DEBUG_printf(("httpGetLength(%p), state = %d\n", http, http->state));
+  DEBUG_printf(("httpGetLength(http=%p), state=%d\n", http, http->state));
 
   if (strcasecmp(http->fields[HTTP_FIELD_TRANSFER_ENCODING], "chunked") == 0)
   {
@@ -1836,7 +1946,7 @@ httpGetLength(http_t *http)	/* I - HTTP data */
     else
       http->data_remaining = atoi(http->fields[HTTP_FIELD_CONTENT_LENGTH]);
 
-    DEBUG_printf(("httpGetLength: content_length = %d\n", http->data_remaining));
+    DEBUG_printf(("httpGetLength: content_length=%d\n", http->data_remaining));
   }
 
   return (http->data_remaining);
@@ -1893,6 +2003,9 @@ http_send(http_t       *http,	/* I - HTTP data */
 				/* Hex digits */
 
 
+  DEBUG_printf(("http_send(http=%p, request=HTTP_%s, uri=\"%s\")\n",
+                http, codes[request], uri));
+
   if (http == NULL || uri == NULL)
     return (-1);
 
@@ -1983,8 +2096,11 @@ http_wait(http_t *http,			/* I - HTTP data */
 #endif /* !WIN32 */
   struct timeval	timeout;	/* Timeout */
   int			nfds;		/* Result from select() */
+  int			set_size;	/* Size of select set */
 
 
+  DEBUG_printf(("http_wait(http=%p, msec=%d)\n", http, msec));
+
  /*
   * Check the SSL/TLS buffers for data first...
   */
@@ -2029,7 +2145,11 @@ http_wait(http_t *http,			/* I - HTTP data */
 
     getrlimit(RLIMIT_NOFILE, &limit);
 
-    http->input_set = calloc(1, (limit.rlim_cur + 31) / 8);
+    set_size = (limit.rlim_cur + 31) / 8 + 4;
+    if (set_size < sizeof(fd_set))
+      set_size = sizeof(fd_set);
+
+    http->input_set = calloc(1, set_size);
 #endif /* WIN32 */
 
     if (!http->input_set)
@@ -2165,6 +2285,8 @@ http_setup_ssl(http_t *http)		/* I - HTTP data */
 #  endif /* HAVE_LIBSSL */
 
 
+  DEBUG_printf(("http_setup_ssl(http=%p)\n", http));
+
 #  ifdef HAVE_LIBSSL
   context = SSL_CTX_new(SSLv23_client_method());
   conn    = SSL_new(context);
@@ -2172,6 +2294,13 @@ http_setup_ssl(http_t *http)		/* I - HTTP data */
   SSL_set_fd(conn, http->fd);
   if (SSL_connect(conn) != 1)
   {
+#    ifdef DEBUG
+    unsigned long	error;	/* Error code */
+
+    while ((error = ERR_get_error()) != 0)
+      printf("http_setup_ssl: %s\n", ERR_error_string(error, NULL));
+#    endif /* DEBUG */
+
     SSL_CTX_free(context);
     SSL_free(conn);
 
@@ -2420,5 +2549,5 @@ CDSAWriteFunc(SSLConnectionRef connection,	/* I  - SSL/TLS connection */
 
 
 /*
- * End of "$Id: http.c,v 1.127 2003/11/19 18:05:55 mike Exp $".
+ * End of "$Id: http.c,v 1.142 2004/08/18 15:36:34 mike Exp $".
  */
