@@ -1,10 +1,10 @@
 /*
- * "$Id: ipp.c,v 1.94 2003/11/07 18:59:18 mike Exp $"
+ * "$Id: ipp.c,v 1.97 2004/06/29 03:44:52 mike Exp $"
  *
  *   Internet Printing Protocol support functions for the Common UNIX
  *   Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -16,9 +16,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636-3142 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -385,7 +385,7 @@ ippAddString(ipp_t      *ipp,			/* I - IPP request */
       if (*p == '_')
         *p = '-';
       else
-        *p = tolower(*p);
+        *p = tolower(*p & 255);
   }
 
   return (attr);
@@ -2557,5 +2557,5 @@ ipp_write_file(int         *fd,			/* I - File descriptor */
 
 
 /*
- * End of "$Id: ipp.c,v 1.94 2003/11/07 18:59:18 mike Exp $".
+ * End of "$Id: ipp.c,v 1.97 2004/06/29 03:44:52 mike Exp $".
  */
