@@ -1,9 +1,9 @@
 /*
- * "$Id: ppd.c,v 1.114 2003/11/17 02:28:14 mike Exp $"
+ * "$Id: ppd.c,v 1.118 2004/06/29 03:44:52 mike Exp $"
  *
  *   PPD file routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636-3142 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -595,11 +595,7 @@ ppdOpen(FILE *fp)			/* I - File to read from */
 	                          cupsLangString(language, CUPS_MSG_GENERAL));
 
           if (group == NULL)
-	  {
-            ppd_status = PPD_ALLOC_ERROR;
-
 	    goto error;
-	  }
 
           DEBUG_printf(("Adding to group %s...\n", group->text));
           option = ppd_get_option(group, keyword);
@@ -832,8 +828,6 @@ ppdOpen(FILE *fp)			/* I - File to read from */
 	  {
 	    DEBUG_puts("Unable to get general group!");
 
-            ppd_status = PPD_ALLOC_ERROR;
-
 	    goto error;
 	  }
 
@@ -979,7 +973,7 @@ ppdOpen(FILE *fp)			/* I - File to read from */
       if (name[0] == '*')
         cups_strcpy(name, name + 1); /* Eliminate leading asterisk */
 
-      for (i = strlen(name) - 1; i > 0 && isspace(name[i]); i --)
+      for (i = strlen(name) - 1; i > 0 && isspace(name[i] & 255); i --)
         name[i] = '\0'; /* Eliminate trailing spaces */
 
       DEBUG_printf(("OpenUI of %s in group %s...\n", name,
@@ -1002,11 +996,7 @@ ppdOpen(FILE *fp)			/* I - File to read from */
 	                        cupsLangString(language, CUPS_MSG_GENERAL));
 
         if (group == NULL)
-	{
-          ppd_status = PPD_ALLOC_ERROR;
-
 	  goto error;
-	}
 
         DEBUG_printf(("Adding to group %s...\n", group->text));
         option = ppd_get_option(group, name);
@@ -1104,11 +1094,7 @@ ppdOpen(FILE *fp)			/* I - File to read from */
       group = ppd_get_group(ppd, "JCL", "JCL");
 
       if (group == NULL)
-      {
-        ppd_status = PPD_ALLOC_ERROR;
-
 	goto error;
-      }
 
      /*
       * Add an option record to the current JCLs...
@@ -1213,6 +1199,9 @@ ppdOpen(FILE *fp)			/* I - File to read from */
 
       group = ppd_get_group(ppd, string, sptr);
 
+      if (group == NULL)
+	goto error;
+
       ppd_free(string);
       string = NULL;
     }
@@ -1938,14 +1927,14 @@ ppd_decode(char *string)		/* I - String to decode */
   outptr = string;
 
   while (*inptr != '\0')
-    if (*inptr == '<' && isxdigit(inptr[1]))
+    if (*inptr == '<' && isxdigit(inptr[1] & 255))
     {
      /*
       * Convert hex to 8-bit values...
       */
 
       inptr ++;
-      while (isxdigit(*inptr))
+      while (isxdigit(*inptr & 255))
       {
 	if (isalpha(*inptr))
 	  *outptr = (tolower(*inptr) - 'a' + 10) << 4;
@@ -1954,6 +1943,9 @@ ppd_decode(char *string)		/* I - String to decode */
 
 	inptr ++;
 
+        if (!isxdigit(*inptr & 255))
+	  break;
+
 	if (isalpha(*inptr))
 	  *outptr |= tolower(*inptr) - 'a' + 10;
 	else
@@ -2113,6 +2105,13 @@ ppd_get_group(ppd_file_t *ppd,		/* I - PPD file */
   {
     DEBUG_printf(("Adding group %s...\n", name));
 
+    if (ppd_conform == PPD_CONFORM_STRICT && strlen(text) >= sizeof(group->text))
+    {
+      ppd_status = PPD_ILLEGAL_TRANSLATION;
+
+      return (NULL);
+    }
+	    
     if (ppd->num_groups == 0)
       group = malloc(sizeof(ppd_group_t));
     else
@@ -2120,7 +2119,11 @@ ppd_get_group(ppd_file_t *ppd,		/* I - PPD file */
 	              (ppd->num_groups + 1) * sizeof(ppd_group_t));
 
     if (group == NULL)
+    {
+      ppd_status = PPD_ALLOC_ERROR;
+
       return (NULL);
+    }
 
     ppd->groups = group;
     group += ppd->num_groups;
@@ -2481,7 +2484,7 @@ ppd_read(FILE *fp,			/* I - File to read from */
       */
 
       for (lineptr = line; *lineptr; lineptr ++)
-        if (!isspace(*lineptr))
+        if (!isspace(*lineptr & 255))
 	  break;
 
       if (*lineptr)
@@ -2501,7 +2504,7 @@ ppd_read(FILE *fp,			/* I - File to read from */
 
     keyptr = keyword;
 
-    while (*lineptr != '\0' && *lineptr != ':' && !isspace(*lineptr))
+    while (*lineptr != '\0' && *lineptr != ':' && !isspace(*lineptr & 255))
     {
       if (*lineptr <= ' ' || *lineptr > 126 || *lineptr == '/' ||
           (keyptr - keyword) >= (PPD_MAX_NAME - 1))
@@ -2522,18 +2525,18 @@ ppd_read(FILE *fp,			/* I - File to read from */
 
 /*    DEBUG_printf(("keyword = \"%s\", lineptr = \"%s\"\n", keyword, lineptr));*/
 
-    if (isspace(*lineptr))
+    if (isspace(*lineptr & 255))
     {
      /*
       * Get an option name...
       */
 
-      while (isspace(*lineptr))
+      while (isspace(*lineptr & 255))
         lineptr ++;
 
       optptr = option;
 
-      while (*lineptr != '\0' && !isspace(*lineptr) && *lineptr != ':' &&
+      while (*lineptr != '\0' && !isspace(*lineptr & 255) && *lineptr != ':' &&
              *lineptr != '/')
       {
 	if (*lineptr <= ' ' || *lineptr > 126 ||
@@ -2548,13 +2551,13 @@ ppd_read(FILE *fp,			/* I - File to read from */
 
       *optptr = '\0';
 
-      if (isspace(*lineptr) && ppd_conform == PPD_CONFORM_STRICT)
+      if (isspace(*lineptr & 255) && ppd_conform == PPD_CONFORM_STRICT)
       {
         ppd_status = PPD_ILLEGAL_WHITESPACE;
 	return (0);
       }
 
-      while (isspace(*lineptr))
+      while (isspace(*lineptr & 255))
 	lineptr ++;
 
       mask |= PPD_OPTION;
@@ -2598,13 +2601,13 @@ ppd_read(FILE *fp,			/* I - File to read from */
 /*      DEBUG_printf(("text = \"%s\", lineptr = \"%s\"\n", text, lineptr));*/
     }
 
-    if (isspace(*lineptr) && ppd_conform == PPD_CONFORM_STRICT)
+    if (isspace(*lineptr & 255) && ppd_conform == PPD_CONFORM_STRICT)
     {
       ppd_status = PPD_ILLEGAL_WHITESPACE;
       return (0);
     }
 
-    while (isspace(*lineptr))
+    while (isspace(*lineptr & 255))
       lineptr ++;
 
     if (*lineptr == ':')
@@ -2614,11 +2617,11 @@ ppd_read(FILE *fp,			/* I - File to read from */
       */
 
       lineptr ++;
-      while (isspace(*lineptr))
+      while (isspace(*lineptr & 255))
         lineptr ++;
 
       strptr = lineptr + strlen(lineptr) - 1;
-      while (strptr >= lineptr && isspace(*strptr))
+      while (strptr >= lineptr && isspace(*strptr & 255))
         *strptr-- = '\0';
 
       if (*strptr == '\"')
@@ -2652,5 +2655,5 @@ ppd_read(FILE *fp,			/* I - File to read from */
 
 
 /*
- * End of "$Id: ppd.c,v 1.114 2003/11/17 02:28:14 mike Exp $".
+ * End of "$Id: ppd.c,v 1.118 2004/06/29 03:44:52 mike Exp $".
  */
