@@ -1,9 +1,9 @@
 /*
- * "$Id: testlang.c,v 1.4 2003/10/30 15:45:20 mike Exp $"
+ * "$Id: testlang.c,v 1.7 2004/08/19 13:54:38 mike Exp $"
  *
- *   HTTP test program for the Common UNIX Printing System (CUPS).
+ *   Localization test program for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products.
+ *   Copyright 1997-2004 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636-3142 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -90,17 +90,30 @@ main(int  argc,				/* I - Number of command-line arguments */
   }
 
   if (language != language2)
+  {
     puts("**** ERROR: Language cache did not work! ****");
+    puts("First result from cupsLangGet:");
+  }
 
   printf("Language = \"%s\"\n", language->language);
   printf("Encoding = \"%s\"\n", charsets[language->encoding]);
   printf("No       = \"%s\"\n", cupsLangString(language, CUPS_MSG_NO));
   printf("Yes      = \"%s\"\n", cupsLangString(language, CUPS_MSG_YES));
 
+  if (language != language2)
+  {
+    puts("Second result from cupsLangGet:");
+
+    printf("Language = \"%s\"\n", language2->language);
+    printf("Encoding = \"%s\"\n", charsets[language2->encoding]);
+    printf("No       = \"%s\"\n", cupsLangString(language2, CUPS_MSG_NO));
+    printf("Yes      = \"%s\"\n", cupsLangString(language2, CUPS_MSG_YES));
+  }
+
   return (0);
 }
 
 
 /*
- * End of "$Id: testlang.c,v 1.4 2003/10/30 15:45:20 mike Exp $".
+ * End of "$Id: testlang.c,v 1.7 2004/08/19 13:54:38 mike Exp $".
  */
