@@ -1,9 +1,9 @@
 /*
- * "$Id: hpgl-input.c,v 1.14 2003/11/07 21:41:38 mike Exp $"
+ * "$Id: hpgl-input.c,v 1.16 2004/02/25 20:14:52 mike Exp $"
  *
  *   HP-GL/2 input processing for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1993-2003 by Easy Software Products.
+ *   Copyright 1993-2004 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -117,7 +117,7 @@ ParseCommand(FILE    *fp,	/* I - File to read from */
 	    }
 
         default : /* HP RTL/PCL control */
-            while ((i = getc(fp)) != EOF && !isupper(i));
+            while ((i = getc(fp)) != EOF && !isupper(i & 255));
             break;
       }
   } while (ch < ' ');
@@ -243,5 +243,5 @@ FreeParameters(int     num_params,	/* I - Number of parameters */
 
 
 /*
- * End of "$Id: hpgl-input.c,v 1.14 2003/11/07 21:41:38 mike Exp $".
+ * End of "$Id: hpgl-input.c,v 1.16 2004/02/25 20:14:52 mike Exp $".
  */
