@@ -1,9 +1,9 @@
 /*
- * "$Id: image.c,v 1.32 2002/12/17 18:59:28 swdev Exp $"
+ * "$Id: image.c,v 1.34 2004/02/25 20:14:52 mike Exp $"
  *
  *   Base image support for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1993-2003 by Easy Software Products.
+ *   Copyright 1993-2004 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -277,13 +277,13 @@ ImageSetMaxTiles(image_t *img,		/* I - Image to set */
           max_size *= 4 * TILE_SIZE * TILE_SIZE;
 	  break;
       case 2 :
-          if (tolower(cache_units[0]) == 'g')
+          if (tolower(cache_units[0] & 255) == 'g')
 	    max_size *= 1024 * 1024 * 1024;
-          else if (tolower(cache_units[0]) == 'm')
+          else if (tolower(cache_units[0] & 255) == 'm')
 	    max_size *= 1024 * 1024;
-	  else if (tolower(cache_units[0]) == 'k')
+	  else if (tolower(cache_units[0] & 255) == 'k')
 	    max_size *= 1024;
-	  else if (tolower(cache_units[0]) == 't')
+	  else if (tolower(cache_units[0] & 255) == 't')
 	    max_size *= 4 * TILE_SIZE * TILE_SIZE;
 	  break;
     }
@@ -771,5 +771,5 @@ flush_tile(image_t *img)	/* I - Image */
 
 
 /*
- * End of "$Id: image.c,v 1.32 2002/12/17 18:59:28 swdev Exp $".
+ * End of "$Id: image.c,v 1.34 2004/02/25 20:14:52 mike Exp $".
  */
