@@ -1,9 +1,9 @@
 /*
- * "$Id: pstops.c,v 1.109 2003/11/24 15:38:46 mike Exp $"
+ * "$Id: pstops.c,v 1.115 2004/06/29 04:14:13 mike Exp $"
  *
  *   PostScript filter for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1993-2003 by Easy Software Products.
+ *   Copyright 1993-2004 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -381,16 +381,14 @@ main(int  argc,			/* I - Number of command-line arguments */
     return (1);
   }
 
-  fwrite(line, 1, len, stdout);
-
  /*
   * Handle leading PJL fun...
   */
 
-  while (!strncmp(line, "\033%-12345X", 9))
+  while (!strncmp(line, "\033%-12345X", 9) || !strncmp(line, "@PJL ", 5))
   {
    /*
-    * Yup, we have leading PJL fun, so copy it until we hit the line
+    * Yup, we have leading PJL fun, so skip it until we hit the line
     * with "ENTER LANGUAGE"...
     */
 
@@ -401,8 +399,6 @@ main(int  argc,			/* I - Number of command-line arguments */
       len = sizeof(line);
       if (psgets(line, &len, fp) == NULL)
         break;
-
-      fwrite(line, 1, len, stdout);
     }
 
     len = sizeof(line);
@@ -421,6 +417,8 @@ main(int  argc,			/* I - Number of command-line arguments */
   * Start sending the document with any commands needed...
   */
 
+  fwrite(line, 1, len, stdout);
+
   saweof      = 0;
   sent_espsp  = 0;
   sent_prolog = 0;
@@ -559,6 +557,12 @@ main(int  argc,			/* I - Number of command-line arguments */
 
         fputs(line, stdout);
 
+	if (!sent_prolog)
+	{
+	  sent_prolog = 1;
+          do_prolog(ppd);
+	}
+
 	if (!sent_setup)
 	{
 	  sent_setup = 1;
@@ -1131,7 +1135,7 @@ check_range(int page)	/* I - Page number */
       if (*range == '-')
       {
         range ++;
-	if (!isdigit(*range))
+	if (!isdigit(*range & 255))
 	  upper = 65535;
 	else
 	  upper = strtol(range, (char **)&range, 10);
@@ -1892,5 +1896,5 @@ start_nup(int number,			/* I - Page number */
 
 
 /*
- * End of "$Id: pstops.c,v 1.109 2003/11/24 15:38:46 mike Exp $".
+ * End of "$Id: pstops.c,v 1.115 2004/06/29 04:14:13 mike Exp $".
  */
