@@ -1,9 +1,9 @@
 /*
- * "$Id: raster.c,v 1.5 2002/12/17 18:59:28 swdev Exp $"
+ * "$Id: raster.c,v 1.8 2004/05/21 23:00:04 mike Exp $"
  *
  *   Raster file routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products.
+ *   Copyright 1997-2004 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -178,9 +178,11 @@ cupsRasterReadPixels(cups_raster_t *r,	/* I - Raster stream */
   {
     bytes = read(r->fd, p, remaining);
 
-    if (bytes <= 0)
+    if (bytes == 0)
+      return (0);
+    else if (bytes < 0)
     {
-      if (errno != EAGAIN && errno != EINTR)
+      if (errno != EINTR)
         return (0);
       else
         continue;
@@ -233,9 +235,11 @@ cupsRasterWritePixels(cups_raster_t *r,	/* I - Raster stream */
   {
     bytes = write(r->fd, p, remaining);
 
-    if (bytes <= 0)
+    if (bytes == 0)
+      return (0);
+    else if (bytes < 0)
     {
-      if (errno != EAGAIN && errno != EINTR)
+      if (errno != EINTR)
         return (0);
       else
         continue;
@@ -250,5 +254,5 @@ cupsRasterWritePixels(cups_raster_t *r,	/* I - Raster stream */
 
 
 /*
- * End of "$Id: raster.c,v 1.5 2002/12/17 18:59:28 swdev Exp $".
+ * End of "$Id: raster.c,v 1.8 2004/05/21 23:00:04 mike Exp $".
  */
