@@ -1,9 +1,9 @@
 /*
- * "$Id: textcommon.c,v 1.29 2003/08/28 14:30:03 mike Exp $"
+ * "$Id: textcommon.c,v 1.32 2004/04/26 19:01:40 mike Exp $"
  *
  *   Common text filter routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products.
+ *   Copyright 1997-2004 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -110,6 +110,7 @@ static char *code_keywords[] =	/* List of known C/C++ keywords... */
 	  "protected",
 	  "public",
 	  "register",
+	  "reinterpret_cast",
 	  "return",
 	  "short",
 	  "signed",
@@ -888,7 +889,7 @@ TextMain(const char *name,	/* I - Name of filter */
 
 	      attr &= ~ATTR_BOLD;
 	    }
-	    else if (!(isalnum(ch) || ch == '_') && keyptr > keyword)
+	    else if (!(isalnum(ch & 255) || ch == '_') && keyptr > keyword)
 	    {
 	     /*
 	      * Look for a keyword...
@@ -914,7 +915,7 @@ TextMain(const char *name,	/* I - Name of filter */
 		}
 	      }
 	    }
-	    else if ((isalnum(ch) || ch == '_') && !ccomment && !cstring)
+	    else if ((isalnum(ch & 255) || ch == '_') && !ccomment && !cstring)
 	    {
 	     /*
 	      * Add characters to the current keyword (if they'll fit).
@@ -1182,5 +1183,5 @@ getutf8(FILE *fp)	/* I - File to read from */
 
 
 /*
- * End of "$Id: textcommon.c,v 1.29 2003/08/28 14:30:03 mike Exp $".
+ * End of "$Id: textcommon.c,v 1.32 2004/04/26 19:01:40 mike Exp $".
  */
