@@ -1,9 +1,9 @@
 /*
- * "$Id: classes.h,v 1.12 2002/12/17 19:00:14 swdev Exp $"
+ * "$Id: classes.h,v 1.15 2004/08/23 18:00:59 mike Exp $"
  *
  *   Printer class definitions for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  */
@@ -36,8 +36,9 @@ extern printer_t	*FindAvailablePrinter(const char *name);
 extern printer_t	*FindClass(const char *name);
 extern void		LoadAllClasses(void);
 extern void		SaveAllClasses(void);
+extern void		UpdateImplicitClasses(void);
 
 
 /*
- * End of "$Id: classes.h,v 1.12 2002/12/17 19:00:14 swdev Exp $".
+ * End of "$Id: classes.h,v 1.15 2004/08/23 18:00:59 mike Exp $".
  */
