@@ -1,9 +1,9 @@
 /*
- * "$Id: client.h,v 1.27 2003/07/20 02:51:54 mike Exp $"
+ * "$Id: client.h,v 1.33 2004/08/23 18:00:59 mike Exp $"
  *
  *   Client definitions for the Common UNIX Printing System (CUPS) scheduler.
  *
- *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  */
@@ -41,6 +41,7 @@ typedef struct
 		*command,		/* Command to run */
 		*options;		/* Options for command */
   int		file;			/* Input/output file */
+  int		file_ready;		/* Input ready on file/pipe? */
   int		pipe_pid;		/* Pipe process ID (or 0 if not a pipe) */
   int		got_fields,		/* Non-zero if all fields seen */
 		field_col;		/* Column within line */
@@ -70,6 +71,8 @@ VAR int			ListenBackLog	VALUE(SOMAXCONN),
 					/* Max backlog of pending connections */
 			LocalPort	VALUE(631);
 					/* Local port to use */
+VAR http_encryption_t	LocalEncryption	VALUE(HTTP_ENCRYPT_IF_REQUESTED);
+					/* Local port encryption to use */
 VAR int			NumListeners	VALUE(0);
 					/* Number of listening sockets */
 VAR listener_t		*Listeners	VALUE(NULL);
@@ -79,6 +82,8 @@ VAR int			NumClients	VALUE(0);
 VAR client_t		*Clients	VALUE(NULL);
 					/* HTTP clients */
 VAR struct sockaddr_in	ServerAddr;	/* Server IP address */
+VAR char		*ServerHeader	VALUE(NULL);
+					/* Server header in requests */
 VAR int			CGIPipes[2]	VALUE2(-1,-1);
 					/* Pipes for CGI error/debug output */
 
@@ -89,7 +94,7 @@ VAR int			CGIPipes[2]	VALUE2(-1,-1);
 
 extern void	AcceptClient(listener_t *lis);
 extern void	CloseAllClients(void);
-extern void	CloseClient(client_t *con);
+extern int	CloseClient(client_t *con);
 extern int	EncryptClient(client_t *con);
 extern int	IsCGI(client_t *con, const char *filename,
 		      struct stat *filestats, mime_type_t *type);
@@ -109,5 +114,5 @@ extern void	UpdateCGI(void);
 extern int	WriteClient(client_t *con);
 
 /*
- * End of "$Id: client.h,v 1.27 2003/07/20 02:51:54 mike Exp $".
+ * End of "$Id: client.h,v 1.33 2004/08/23 18:00:59 mike Exp $".
  */
