@@ -1,9 +1,9 @@
 /*
- * "$Id: conf.c,v 1.140 2003/11/07 19:41:25 mike Exp $"
+ * "$Id: conf.c,v 1.151 2004/08/23 18:00:59 mike Exp $"
  *
  *   Configuration routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -39,6 +39,7 @@
 #include <stdarg.h>
 #include <pwd.h>
 #include <grp.h>
+#include <sys/utsname.h>
 
 #ifdef HAVE_CDSASSL
 #  include <Security/SecureTransport.h>
@@ -124,6 +125,7 @@ static var_t	variables[] =
   { "PreserveJobHistory",	&JobHistory,		VAR_BOOLEAN },
   { "Printcap",			&Printcap,		VAR_STRING },
   { "PrintcapGUI",		&PrintcapGUI,		VAR_STRING },
+  { "ReloadTimeout",		&ReloadTimeout,		VAR_INTEGER },
   { "RemoteRoot",		&RemoteRoot,		VAR_STRING },
   { "RequestRoot",		&RequestRoot,		VAR_STRING },
   { "RIPCache",			&RIPCache,		VAR_STRING },
@@ -176,7 +178,6 @@ ReadConfiguration(void)
   char		*language;		/* Language string */
   struct passwd	*user;			/* Default user */
   struct group	*group;			/* Default group */
-  int		run_user;		/* User that will be running cupsd */
   char		*old_serverroot,	/* Old ServerRoot */
 		*old_requestroot;	/* Old RequestRoot */
 
@@ -252,6 +253,7 @@ ReadConfiguration(void)
   SetString(&PrintcapGUI, "/usr/bin/glpoptions");
   SetString(&FontPath, CUPS_FONTPATH);
   SetString(&RemoteRoot, "remroot");
+  SetString(&ServerHeader, "CUPS/1.1");
 
   strlcpy(temp, ConfigurationFile, sizeof(temp));
   if ((slash = strrchr(temp, '/')) != NULL)
@@ -354,6 +356,7 @@ ReadConfiguration(void)
   MaxLogSize          = 1024 * 1024;
   MaxPrinterHistory   = 10;
   MaxRequestSize      = 0;
+  ReloadTimeout	      = 60;
   RootCertDuration    = 300;
   RunAsUser           = FALSE;
   Timeout             = DEFAULT_TIMEOUT;
@@ -388,9 +391,9 @@ ReadConfiguration(void)
     return (0);
 
   if (RunAsUser)
-    run_user = User;
+    RunUser = User;
   else
-    run_user = getuid();
+    RunUser = getuid();
 
  /*
   * Use the default system group if none was supplied in cupsd.conf...
@@ -422,6 +425,26 @@ ReadConfiguration(void)
 
   LogMessage(L_INFO, "Loaded configuration file \"%s\"", ConfigurationFile);
 
+ /*
+  * Check that we have at least one listen/port line; if not, report this
+  * as an error and exit!
+  */
+
+  if (NumListeners == 0)
+  {
+   /*
+    * No listeners!
+    */
+
+    LogMessage(L_EMERG, "No valid Listen or Port lines were found in the configuration file!");
+
+   /*
+    * Commit suicide...
+    */
+
+    kill(getpid(), SIGTERM);
+  }
+
  /*
   * Set the default locale using the language and charset...
   */
@@ -446,13 +469,13 @@ ReadConfiguration(void)
     SetStringf(&ServerCertificate, "%s/%s", ServerRoot, ServerCertificate);
 
 #  if defined(HAVE_LIBSSL) || defined(HAVE_GNUTLS)
-  chown(ServerCertificate, run_user, Group);
+  chown(ServerCertificate, RunUser, Group);
   chmod(ServerCertificate, ConfigFilePerm);
 
   if (ServerKey[0] != '/')
     SetStringf(&ServerKey, "%s/%s", ServerRoot, ServerKey);
 
-  chown(ServerKey, run_user, Group);
+  chown(ServerKey, RunUser, Group);
   chmod(ServerKey, ConfigFilePerm);
 #  endif /* HAVE_LIBSSL || HAVE_GNUTLS */
 #endif /* HAVE_SSL */
@@ -462,27 +485,27 @@ ReadConfiguration(void)
   * writable by the user and group in the cupsd.conf file...
   */
 
-  chown(ServerRoot, run_user, Group);
+  chown(ServerRoot, RunUser, Group);
   chmod(ServerRoot, 0775);
 
   snprintf(temp, sizeof(temp), "%s/certs", ServerRoot);
-  chown(temp, run_user, Group);
+  chown(temp, RunUser, Group);
   chmod(temp, 0711);
 
   snprintf(temp, sizeof(temp), "%s/ppd", ServerRoot);
-  chown(temp, run_user, Group);
+  chown(temp, RunUser, Group);
   chmod(temp, 0755);
 
   snprintf(temp, sizeof(temp), "%s/ssl", ServerRoot);
-  chown(temp, run_user, Group);
+  chown(temp, RunUser, Group);
   chmod(temp, 0700);
 
   snprintf(temp, sizeof(temp), "%s/cupsd.conf", ServerRoot);
-  chown(temp, run_user, Group);
+  chown(temp, RunUser, Group);
   chmod(temp, ConfigFilePerm);
 
   snprintf(temp, sizeof(temp), "%s/classes.conf", ServerRoot);
-  chown(temp, run_user, Group);
+  chown(temp, RunUser, Group);
 #ifdef __APPLE__
   chmod(temp, 0600);
 #else
@@ -490,7 +513,7 @@ ReadConfiguration(void)
 #endif /* __APPLE__ */
 
   snprintf(temp, sizeof(temp), "%s/printers.conf", ServerRoot);
-  chown(temp, run_user, Group);
+  chown(temp, RunUser, Group);
 #ifdef __APPLE__
   chmod(temp, 0600);
 #else
@@ -506,7 +529,7 @@ ReadConfiguration(void)
   * permissions...
   */
 
-  chown(RequestRoot, run_user, Group);
+  chown(RequestRoot, RunUser, Group);
   chmod(RequestRoot, 0710);
 
   if (strncmp(TempDir, RequestRoot, strlen(RequestRoot)) == 0)
@@ -516,7 +539,7 @@ ReadConfiguration(void)
     * is under the spool directory...
     */
 
-    chown(TempDir, run_user, Group);
+    chown(TempDir, RunUser, Group);
     chmod(TempDir, 01770);
   }
 
@@ -650,6 +673,8 @@ ReadConfiguration(void)
     LoadAllPrinters();
     LoadAllClasses();
 
+    CreateCommonData();
+
    /*
     * Load devices and PPDs...
     */
@@ -669,7 +694,11 @@ ReadConfiguration(void)
     LogMessage(L_INFO, "Full reload complete.");
   }
   else
+  {
+    CreateCommonData();
+
     LogMessage(L_INFO, "Partial reload complete.");
+  }
 
  /*
   * Reset the reload state...
@@ -812,7 +841,7 @@ read_configuration(cups_file_t *fp)	/* I - File to read from */
       }
       else
       {
-        LogMessage(L_ERROR, "ReadConfiguration() Syntax error on line %d.",
+        LogMessage(L_ERROR, "Syntax error on line %d.",
 	           linenum);
         return (0);
       }
@@ -1199,9 +1228,13 @@ read_configuration(cups_file_t *fp)	/* I - File to read from */
 	if (value[0] == '*')
 	  value ++;
 
+        strlcpy(name, value, sizeof(name));
+	if ((nameptr = strchr(name, ' ')) != NULL)
+	  *nameptr = '\0';
+
         relay->from.type             = AUTH_NAME;
-	relay->from.mask.name.name   = strdup(value);
-	relay->from.mask.name.length = strlen(value);
+	relay->from.mask.name.name   = strdup(name);
+	relay->from.mask.name.length = strlen(name);
       }
       else
       {
@@ -1357,7 +1390,7 @@ read_configuration(cups_file_t *fp)	/* I - File to read from */
 	if (p != NULL)
 	  User = p->pw_uid;
 	else
-	  LogMessage(L_WARN, "ReadConfiguration() Unknown username \"%s\"",
+	  LogMessage(L_WARN, "Unknown username \"%s\"",
 	             value);
       }
     }
@@ -1379,7 +1412,7 @@ read_configuration(cups_file_t *fp)	/* I - File to read from */
 	if (g != NULL)
 	  Group = g->gr_gid;
 	else
-	  LogMessage(L_WARN, "ReadConfiguration() Unknown groupname \"%s\"",
+	  LogMessage(L_WARN, "Unknown groupname \"%s\"",
 	             value);
       }
     }
@@ -1389,14 +1422,34 @@ read_configuration(cups_file_t *fp)	/* I - File to read from */
       * System (admin) group(s)...
       */
 
-      char *valueptr; /* Pointer into value */
+      char	*valueptr,	/* Pointer into value */
+		quote;		/* Quote character */
 
 
       for (i = NumSystemGroups; *value && i < MAX_SYSTEM_GROUPS; i ++)
       {
-        for (valueptr = value; *valueptr; valueptr ++)
-	  if (isspace(*valueptr) || *valueptr == ',')
-	    break;
+        if (*value == '\'' || *value == '\"')
+	{
+	 /*
+	  * Scan quoted name...
+	  */
+
+	  quote = *value++;
+
+	  for (valueptr = value; *valueptr; valueptr ++)
+	    if (*valueptr == quote)
+	      break;
+	}
+	else
+	{
+	 /*
+	  * Scan space or comma-delimited name...
+	  */
+
+          for (valueptr = value; *valueptr; valueptr ++)
+	    if (isspace(*valueptr) || *valueptr == ',')
+	      break;
+        }
 
         if (*valueptr)
           *valueptr++ = '\0';
@@ -1425,7 +1478,7 @@ read_configuration(cups_file_t *fp)	/* I - File to read from */
       else if (strcasecmp(value, "double") == 0)
         HostNameLookups = 2;
       else
-	LogMessage(L_WARN, "ReadConfiguration() Unknown HostNameLookups %s on line %d.",
+	LogMessage(L_WARN, "Unknown HostNameLookups %s on line %d.",
 	           value, linenum);
     }
     else if (strcasecmp(name, "LogLevel") == 0)
@@ -1468,9 +1521,37 @@ read_configuration(cups_file_t *fp)	/* I - File to read from */
       else if (strcasecmp(value, "solaris") == 0)
         PrintcapFormat = PRINTCAP_SOLARIS;
       else
-	LogMessage(L_WARN, "ReadConfiguration() Unknown PrintcapFormat %s on line %d.",
+	LogMessage(L_WARN, "Unknown PrintcapFormat %s on line %d.",
 	           value, linenum);
     }
+    else if (!strcasecmp(name, "ServerTokens"))
+    {
+     /*
+      * Set the string used for the Server header...
+      */
+
+      struct utsname plat;		/* Platform info */
+
+
+      uname(&plat);
+
+      if (!strcasecmp(value, "ProductOnly"))
+        SetString(&ServerHeader, "CUPS");
+      else if (!strcasecmp(value, "Major"))
+        SetString(&ServerHeader, "CUPS/1");
+      else if (!strcasecmp(value, "Minor"))
+        SetString(&ServerHeader, "CUPS/1.1");
+      else if (!strcasecmp(value, "Minimal"))
+        SetString(&ServerHeader, CUPS_MINIMAL);
+      else if (!strcasecmp(value, "OS"))
+        SetStringf(&ServerHeader, CUPS_MINIMAL " (%s)", plat.sysname);
+      else if (!strcasecmp(value, "Full"))
+        SetStringf(&ServerHeader, CUPS_MINIMAL " (%s) IPP/1.1", plat.sysname);
+      else if (!strcasecmp(value, "None"))
+        ClearString(&ServerHeader);
+      else
+        LogMessage(L_WARN, "Unknown ServerTokens %s on line %d.", value, linenum);
+    }
     else
     {
      /*
@@ -1504,13 +1585,13 @@ read_configuration(cups_file_t *fp)	/* I - File to read from */
 
 	      if (units && *units)
 	      {
-        	if (tolower(units[0]) == 'g')
+        	if (tolower(units[0] & 255) == 'g')
 		  n *= 1024 * 1024 * 1024;
-        	else if (tolower(units[0]) == 'm')
+        	else if (tolower(units[0] & 255) == 'm')
 		  n *= 1024 * 1024;
-		else if (tolower(units[0]) == 'k')
+		else if (tolower(units[0] & 255) == 'k')
 		  n *= 1024;
-		else if (tolower(units[0]) == 't')
+		else if (tolower(units[0] & 255) == 't')
 		  n *= 262144;
 	      }
 
@@ -1602,7 +1683,7 @@ read_location(cups_file_t *fp,		/* I - Configuration file */
 
     len = strlen(line);
 
-    while (len > 0 && isspace(line[len - 1]))
+    while (len > 0 && isspace(line[len - 1] & 255))
     {
       len --;
       line[len] = '\0';
@@ -1612,14 +1693,14 @@ read_location(cups_file_t *fp,		/* I - Configuration file */
     * Extract the name from the beginning of the line...
     */
 
-    for (value = line; isspace(*value); value ++);
+    for (value = line; isspace(*value & 255); value ++);
 
-    for (nameptr = name; *value != '\0' && !isspace(*value) &&
+    for (nameptr = name; *value != '\0' && !isspace(*value & 255) &&
                              nameptr < (name + sizeof(name) - 1);)
       *nameptr++ = *value++;
     *nameptr = '\0';
 
-    while (isspace(*value))
+    while (isspace(*value & 255))
       value ++;
 
     if (name[0] == '\0')
@@ -1641,7 +1722,7 @@ read_location(cups_file_t *fp,		/* I - Configuration file */
       while (*value)
       {
         for (valptr = value;
-	     !isspace(*valptr) && *valptr != '>' && *valptr;
+	     !isspace(*valptr & 255) && *valptr != '>' && *valptr;
 	     valptr ++);
 
 	if (*valptr)
@@ -1665,7 +1746,7 @@ read_location(cups_file_t *fp,		/* I - Configuration file */
 	  LogMessage(L_WARN, "Unknown request type %s on line %d!", value,
 	             linenum);
 
-        for (value = valptr; isspace(*value) || *value == '>'; value ++);
+        for (value = valptr; isspace(*value & 255) || *value == '>'; value ++);
       }
 
       if (strcasecmp(name, "<LimitExcept") == 0)
@@ -1728,7 +1809,7 @@ read_location(cups_file_t *fp,		/* I - Configuration file */
 
 	value += 4;
 
-	while (isspace(*value))
+	while (isspace(*value & 255))
 	  value ++;
       }
 
@@ -1770,7 +1851,7 @@ read_location(cups_file_t *fp,		/* I - Configuration file */
 	else
 	  DenyIP(loc, ~0, 0);
       }
-      else if (value[0] == '*' || value[0] == '.' || !isdigit(value[0]))
+      else if (value[0] == '*' || value[0] == '.' || !isdigit(value[0] & 255))
       {
        /*
         * Host or domain name...
@@ -1915,7 +1996,7 @@ read_location(cups_file_t *fp,		/* I - Configuration file */
       */
 
       for (valptr = value;
-	   !isspace(*valptr) && *valptr != '>' && *valptr;
+	   !isspace(*valptr & 255) && *valptr != '>' && *valptr;
 	   valptr ++);
 
       if (*valptr)
@@ -1939,14 +2020,14 @@ read_location(cups_file_t *fp,		/* I - Configuration file */
 
       for (value = valptr; *value;)
       {
-        for (valptr = value; !isspace(*valptr) && *valptr; valptr ++);
+        for (valptr = value; !isspace(*valptr & 255) && *valptr; valptr ++);
 
 	if (*valptr)
 	  *valptr++ = '\0';
 
         AddName(loc, value);
 
-        for (value = valptr; isspace(*value); value ++);
+        for (value = valptr; isspace(*value & 255); value ++);
       }
     }
     else if (strcasecmp(name, "Satisfy") == 0)
@@ -2038,7 +2119,7 @@ get_address(char               *value,		/* I - Value string */
 
   if (portname[0] != '\0')
   {
-    if (isdigit(portname[0]))
+    if (isdigit(portname[0] & 255))
       address->sin_port = htons(atoi(portname));
     else
     {
@@ -2147,5 +2228,5 @@ CDSAGetServerCerts(void)
 
 
 /*
- * End of "$Id: conf.c,v 1.140 2003/11/07 19:41:25 mike Exp $".
+ * End of "$Id: conf.c,v 1.151 2004/08/23 18:00:59 mike Exp $".
  */
