@@ -1,10 +1,10 @@
 /*
- * "$Id: conf.h,v 1.60 2003/10/09 19:13:30 mike Exp $"
+ * "$Id: conf.h,v 1.64 2004/08/23 18:00:59 mike Exp $"
  *
  *   Configuration file definitions for the Common UNIX Printing System (CUPS)
  *   scheduler.
  *
- *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -16,9 +16,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  */
@@ -146,10 +146,13 @@ VAR int			ClassifyOverride	VALUE(0),
 					/* Current filter level */
 			FilterNice		VALUE(0),
 					/* Nice value for filters */
+			ReloadTimeout		VALUE(0),
+					/* Timeout before reload from SIGHUP */
 			RootCertDuration	VALUE(300),
 					/* Root certificate update interval */
 			RunAsUser		VALUE(FALSE),
 					/* Run as unpriviledged user? */
+			RunUser,	/* User to run as, used for files */
 			PrintcapFormat		VALUE(PRINTCAP_BSD);
 					/* Format of printcap file? */
 VAR cups_file_t		*AccessFile		VALUE(NULL),
@@ -194,5 +197,5 @@ extern int	LogPage(job_t *job, const char *page);
 
 
 /*
- * End of "$Id: conf.h,v 1.60 2003/10/09 19:13:30 mike Exp $".
+ * End of "$Id: conf.h,v 1.64 2004/08/23 18:00:59 mike Exp $".
  */
