@@ -1,9 +1,9 @@
 /*
- * "$Id: cups-polld.c,v 1.20 2003/04/29 19:34:26 mike Exp $"
+ * "$Id: cups-polld.c,v 1.25 2004/08/23 18:36:50 mike Exp $"
  *
  *   Polling daemon for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -31,6 +31,7 @@
  * Include necessary headers...
  */
 
+#include <cups/http-private.h>
 #include <cups/cups.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -38,34 +39,6 @@
 #include <cups/string.h>
 
 
-/*
- * Some OS's don't have hstrerror(), most notably Solaris...
- */
-
-#ifndef HAVE_HSTRERROR
-#  define hstrerror cups_hstrerror
-
-const char *					/* O - Error string */
-cups_hstrerror(int error)			/* I - Error number */
-{
-  static const char * const errors[] =
-		{
-		  "OK",
-		  "Host not found.",
-		  "Try again.",
-		  "Unrecoverable lookup error.",
-		  "No data associated with name."
-		};
-
-
-  if (error < 0 || error > 4)
-    return ("Unknown hostname lookup error.");
-  else
-    return (errors[error]);
-}
-#endif /* !HAVE_HSTRERROR */
-
-
 /*
  * Local functions...
  */
@@ -186,8 +159,6 @@ main(int  argc,				/* I - Number of command-line arguments */
     if (remain > 0) 
       sleep(remain);
   }
-
-  return (0);
 }
 
 
@@ -216,11 +187,13 @@ poll_server(http_t      *http,		/* I - HTTP connection */
 			*make_model;	/* printer-make-and-model */
   cups_ptype_t		type;		/* printer-type */
   ipp_pstate_t		state;		/* printer-state */
+  int			accepting;	/* printer-is-accepting-jobs */
   struct sockaddr_in	addr;		/* Broadcast address */
   char			packet[1540];	/* Data packet */
   static const char * const attrs[] =	/* Requested attributes */
 			{
 			  "printer-info",
+			  "printer-is-accepting-jobs",
 			  "printer-location",
 			  "printer-make-and-model",
 			  "printer-name",
@@ -316,6 +289,7 @@ poll_server(http_t      *http,		/* I - HTTP connection */
       location   = "";
       make_model = "";
       type       = CUPS_PRINTER_REMOTE;
+      accepting  = 1;
       state      = IPP_PRINTER_IDLE;
 
       while (attr != NULL && attr->group_tag == IPP_TAG_PRINTER)
@@ -328,6 +302,10 @@ poll_server(http_t      *http,		/* I - HTTP connection */
 	    attr->value_tag == IPP_TAG_TEXT)
 	  info = attr->values[0].string.text;
 
+        if (strcmp(attr->name, "printer-is-accepting-jobs") == 0 &&
+	    attr->value_tag == IPP_TAG_BOOLEAN)
+	  accepting = attr->values[0].boolean;
+
         if (strcmp(attr->name, "printer-location") == 0 &&
 	    attr->value_tag == IPP_TAG_TEXT)
 	  location = attr->values[0].string.text;
@@ -369,9 +347,13 @@ poll_server(http_t      *http,		/* I - HTTP connection */
 	* Send the printer information...
 	*/
 
+        type |= CUPS_PRINTER_REMOTE;
+
+	if (!accepting)
+	  type |= CUPS_PRINTER_REJECTING;
+
 	snprintf(packet, sizeof(packet), "%x %x %s \"%s\" \"%s\" \"%s\"\n",
-        	 type | CUPS_PRINTER_REMOTE, state, uri,
-		 location, info, make_model);
+        	 type, state, uri, location, info, make_model);
 
         fprintf(stderr, "DEBUG2: %s Sending %s", prefix, packet);
 
@@ -423,5 +405,5 @@ poll_server(http_t      *http,		/* I - HTTP connection */
 
 
 /*
- * End of "$Id: cups-polld.c,v 1.20 2003/04/29 19:34:26 mike Exp $".
+ * End of "$Id: cups-polld.c,v 1.25 2004/08/23 18:36:50 mike Exp $".
  */
