@@ -1,9 +1,9 @@
 /*
- * "$Id: cupsd.h,v 1.51 2003/09/02 20:30:47 mike Exp $"
+ * "$Id: cupsd.h,v 1.54 2004/09/09 15:10:18 mike Exp $"
  *
  *   Main header file for the Common UNIX Printing System (CUPS) scheduler.
  *
- *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  */
@@ -206,8 +206,9 @@ extern void	SetString(char **s, const char *v);
 extern void	SetStringf(char **s, const char *f, ...);
 extern void	StartServer(void);
 extern void	StopServer(void);
+extern int	cupsdPipe(int *fds);
 
 
 /*
- * End of "$Id: cupsd.h,v 1.51 2003/09/02 20:30:47 mike Exp $".
+ * End of "$Id: cupsd.h,v 1.54 2004/09/09 15:10:18 mike Exp $".
  */
