@@ -1,9 +1,9 @@
 /*
- * "$Id: devices.c,v 1.18 2002/12/17 19:00:15 swdev Exp $"
+ * "$Id: devices.c,v 1.21 2004/06/29 03:27:35 mike Exp $"
  *
  *   Device scanning routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products.
+ *   Copyright 1997-2004 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,15 +15,17 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636-3142 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
  * Contents:
  *
- *   LoadDevices() - Load all available devices.
+ *   LoadDevices()     - Load all available devices.
+ *   compare_devs()    - Compare PPD file make and model names for sorting.
+ *   sigalrm_handler() - Handle alarm signals for backends that get hung
  */
 
 /*
@@ -342,12 +344,12 @@ LoadDevices(const char *d)	/* I - Directory to scan */
 
 
 /*
- * 'compare_devs()' - Compare PPD file make and model names for sorting.
+ * 'compare_devs()' - Compare device names for sorting.
  */
 
 static int				/* O - Result of comparison */
-compare_devs(const dev_info_t *d0,	/* I - First PPD file */
-             const dev_info_t *d1)	/* I - Second PPD file */
+compare_devs(const dev_info_t *d0,	/* I - First device */
+             const dev_info_t *d1)	/* I - Second device */
 {
   const char	*s,			/* First name */
 		*t;			/* Second name */
@@ -369,7 +371,7 @@ compare_devs(const dev_info_t *d0,	/* I - First PPD file */
 
   while (*s && *t)
   {
-    if (isdigit(*s) && isdigit(*t))
+    if (isdigit(*s & 255) && isdigit(*t & 255))
     {
      /*
       * Got a number; start by skipping leading 0's...
@@ -384,7 +386,7 @@ compare_devs(const dev_info_t *d0,	/* I - First PPD file */
       * Skip equal digits...
       */
 
-      while (isdigit(*s) && *s == *t)
+      while (isdigit(*s & 255) && *s == *t)
       {
         s ++;
 	t ++;
@@ -394,11 +396,11 @@ compare_devs(const dev_info_t *d0,	/* I - First PPD file */
       * Bounce out if *s and *t aren't both digits...
       */
 
-      if (isdigit(*s) && !isdigit(*t))
+      if (isdigit(*s & 255) && !isdigit(*t & 255))
         return (1);
-      else if (!isdigit(*s) && isdigit(*t))
+      else if (!isdigit(*s & 255) && isdigit(*t & 255))
         return (-1);
-      else if (!isdigit(*s) || !isdigit(*t))
+      else if (!isdigit(*s & 255) || !isdigit(*t & 255))
         continue;     
 
       if (*s < *t)
@@ -414,13 +416,13 @@ compare_devs(const dev_info_t *d0,	/* I - First PPD file */
       s ++;
       t ++;
 
-      while (isdigit(*s))
+      while (isdigit(*s & 255))
       {
         digits ++;
 	s ++;
       }
 
-      while (isdigit(*t))
+      while (isdigit(*t & 255))
       {
         digits --;
 	t ++;
@@ -478,5 +480,5 @@ sigalrm_handler(int sig)	/* I - Signal number */
 
 
 /*
- * End of "$Id: devices.c,v 1.18 2002/12/17 19:00:15 swdev Exp $".
+ * End of "$Id: devices.c,v 1.21 2004/06/29 03:27:35 mike Exp $".
  */
