@@ -1,9 +1,9 @@
 /*
- * "$Id: dirsvc.c,v 1.126 2003/10/22 14:41:33 mike Exp $"
+ * "$Id: dirsvc.c,v 1.137 2004/09/09 15:10:18 mike Exp $"
  *
  *   Directory services routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636-3142 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -205,7 +205,7 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
       * Force the URI to point to the real server...
       */
 
-      p->type      = type;
+      p->type      = type & ~CUPS_PRINTER_REJECTING;
       p->accepting = 1;
       SetString(&p->uri, uri);
       SetString(&p->device_uri, uri);
@@ -279,7 +279,7 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
       * Force the URI to point to the real server...
       */
 
-      p->type      = type;
+      p->type      = type & ~CUPS_PRINTER_REJECTING;
       p->accepting = 1;
       SetString(&p->hostname, host);
       SetString(&p->uri, uri);
@@ -296,6 +296,22 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
   p->state       = state;
   p->browse_time = time(NULL);
 
+  if (type & CUPS_PRINTER_REJECTING)
+  {
+    type &= ~CUPS_PRINTER_REJECTING;
+
+    if (p->accepting)
+    {
+      update       = 1;
+      p->accepting = 0;
+    }
+  }
+  else if (!p->accepting)
+  {
+    update       = 1;
+    p->accepting = 1;
+  }
+
   if (p->type != type)
   {
     p->type = type;
@@ -334,7 +350,10 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
   }
 
   if (update)
+  {
     SetPrinterAttrs(p);
+    UpdateImplicitClasses();
+  }
 
  /*
   * See if we have a default printer...  If not, make the first printer the
@@ -492,11 +511,11 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
 void
 SendBrowseList(void)
 {
-  int			count;	/* Number of dests to update */
-  printer_t		*p,	/* Current printer */
-			*np;	/* Next printer */
-  time_t		ut,	/* Minimum update time */
-			to;	/* Timeout time */
+  int			count;		/* Number of dests to update */
+  printer_t		*p,		/* Current printer */
+			*np;		/* Next printer */
+  time_t		ut,		/* Minimum update time */
+			to;		/* Timeout time */
 
 
   if (!Browsing || !BrowseProtocols)
@@ -515,20 +534,65 @@ SendBrowseList(void)
 
   if (BrowseInterval > 0)
   {
-    for (count = 0, p = Printers; p != NULL; p = p->next)
+    int	max_count;			/* Maximum number to update */
+
+
+   /*
+    * Throttle the number of printers we'll be updating this time
+    * around based on the number of queues that need updating and
+    * the maximum number of queues to update each second...
+    */
+
+    max_count = 2 * NumPrinters / BrowseInterval + 1;
+
+    for (count = 0, p = Printers; count < max_count && p != NULL; p = p->next)
       if (!(p->type & (CUPS_PRINTER_REMOTE | CUPS_PRINTER_IMPLICIT)) &&
           p->browse_time < ut)
         count ++;
 
    /*
-    * Throttle the number of printers we'll be updating this time
-    * around...
+    * Loop through all of the printers and send local updates as needed...
     */
 
-    count = 2 * count / BrowseInterval + 1;
+    for (p = BrowseNext; count > 0; p = p->next)
+    {
+     /*
+      * Check for wraparound...
+      */
+
+      if (!p)
+        p = Printers;
+
+      if (!p)
+        break;
+      else if (p->type & (CUPS_PRINTER_REMOTE | CUPS_PRINTER_IMPLICIT))
+        continue;
+      else if (p->browse_time < ut)
+      {
+       /*
+	* Need to send an update...
+	*/
+
+	count --;
+
+	p->browse_time = time(NULL);
+
+	if (BrowseProtocols & BROWSE_CUPS)
+          SendCUPSBrowse(p);
+
+#ifdef HAVE_LIBSLP
+	if (BrowseProtocols & BROWSE_SLP)
+          SendSLPBrowse(p);
+#endif /* HAVE_LIBSLP */
+      }
+    }
+
+   /*
+    * Save where we left off so that all printers get updated...
+    */
+
+    BrowseNext = p;
   }
-  else
-    count = 0;
 
  /*
   * Loop through all of the printers and send local updates as needed...
@@ -536,14 +600,18 @@ SendBrowseList(void)
 
   for (p = Printers; p != NULL; p = np)
   {
+   /*
+    * Save the next printer pointer...
+    */
+
     np = p->next;
 
+   /*
+    * If this is a remote queue, see if it needs to be timed out...
+    */
+
     if (p->type & CUPS_PRINTER_REMOTE)
     {
-     /*
-      * See if this printer needs to be timed out...
-      */
-
       if (p->browse_time < to)
       {
         LogMessage(L_INFO, "Remote destination \"%s\" has timed out; deleting it...",
@@ -551,25 +619,6 @@ SendBrowseList(void)
         DeletePrinter(p, 1);
       }
     }
-    else if (p->browse_time < ut && count > 0 &&
-             !(p->type & CUPS_PRINTER_IMPLICIT))
-    {
-     /*
-      * Need to send an update...
-      */
-
-      count --;
-
-      p->browse_time = time(NULL);
-
-      if (BrowseProtocols & BROWSE_CUPS)
-        SendCUPSBrowse(p);
-
-#ifdef HAVE_LIBSLP
-      if (BrowseProtocols & BROWSE_SLP)
-        SendSLPBrowse(p);
-#endif /* HAVE_LIBSLP */
-    }
   }
 }
 
@@ -582,12 +631,22 @@ void
 SendCUPSBrowse(printer_t *p)		/* I - Printer to send */
 {
   int			i;		/* Looping var */
+  cups_ptype_t		type;		/* Printer type */
   dirsvc_addr_t		*b;		/* Browse address */
   int			bytes;		/* Length of packet */
   char			packet[1453];	/* Browse data packet */
   cups_netif_t		*iface;		/* Network interface */
 
 
+ /*
+  * Figure out the printer type value...
+  */
+
+  type = p->type | CUPS_PRINTER_REMOTE;
+
+  if (!p->accepting)
+    type |= CUPS_PRINTER_REJECTING;
+
  /*
   * Send a packet to each browse address...
   */
@@ -613,11 +672,11 @@ SendCUPSBrowse(printer_t *p)		/* I - Printer to send */
 	  * Only send to local interfaces...
 	  */
 
-	  if (!iface->is_local)
+	  if (!iface->is_local || !iface->port)
 	    continue;
 
-	  snprintf(packet, sizeof(packet), "%x %x ipp://%s/%s/%s \"%s\" \"%s\" \"%s\"\n",
-        	   p->type | CUPS_PRINTER_REMOTE, p->state, iface->hostname,
+	  snprintf(packet, sizeof(packet), "%x %x ipp://%s:%d/%s/%s \"%s\" \"%s\" \"%s\"\n",
+        	   type, p->state, iface->hostname, iface->port,
 		   (p->type & CUPS_PRINTER_CLASS) ? "classes" : "printers",
 		   p->name, p->location ? p->location : "",
 		   p->info ? p->info : "",
@@ -641,8 +700,11 @@ SendCUPSBrowse(printer_t *p)		/* I - Printer to send */
         * Send to the named interface...
 	*/
 
-	snprintf(packet, sizeof(packet), "%x %x ipp://%s/%s/%s \"%s\" \"%s\" \"%s\"\n",
-        	 p->type | CUPS_PRINTER_REMOTE, p->state, iface->hostname,
+	if (!iface->port)
+	  continue;
+
+	snprintf(packet, sizeof(packet), "%x %x ipp://%s:%d/%s/%s \"%s\" \"%s\" \"%s\"\n",
+        	 type, p->state, iface->hostname, iface->port,
 		 (p->type & CUPS_PRINTER_CLASS) ? "classes" : "printers",
 		 p->name, p->location ? p->location : "",
 		 p->info ? p->info : "",
@@ -668,7 +730,7 @@ SendCUPSBrowse(printer_t *p)		/* I - Printer to send */
       */
 
       snprintf(packet, sizeof(packet), "%x %x %s \"%s\" \"%s\" \"%s\"\n",
-               p->type | CUPS_PRINTER_REMOTE, p->state, p->uri,
+       	       type, p->state, p->uri,
 	       p->location ? p->location : "",
 	       p->info ? p->info : "",
 	       p->make_model ? p->make_model : "Unknown");
@@ -772,6 +834,12 @@ StartBrowsing(void)
       return;
     }
 
+   /*
+    * Close the socket on exec...
+    */
+
+    fcntl(BrowseSocket, F_SETFD, fcntl(BrowseSocket, F_GETFD) | FD_CLOEXEC);
+
    /*
     * Finally, add the socket to the input selection set...
     */
@@ -817,7 +885,6 @@ StartPolling(void)
   char		bport[10];	/* Browser port */
   char		interval[10];	/* Poll interval */
   int		statusfds[2];	/* Status pipe */
-  int		fd;		/* Current file descriptor */
 #if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
   struct sigaction action;	/* POSIX signal handler */
 #endif /* HAVE_SIGACTION && !HAVE_SIGSET */
@@ -849,7 +916,7 @@ StartPolling(void)
   * polling daemon...
   */
 
-  if (pipe(statusfds))
+  if (cupsdPipe(statusfds))
   {
     LogMessage(L_ERROR, "Unable to create polling status pipes - %s.",
 	       strerror(errno));
@@ -917,9 +984,6 @@ StartPolling(void)
       close(2);
       dup(statusfds[1]);
 
-      for (fd = 3; fd < MaxFDs; fd ++)
-	close(fd);
-
      /*
       * Unblock signals before doing the exec...
       */
@@ -1064,14 +1128,14 @@ UpdateCUPSBrowse(void)
   int		auth;			/* Authorization status */
   int		len;			/* Length of name string */
   int		bytes;			/* Number of bytes left */
-  char		packet[1540],		/* Broadcast packet */
+  char		packet[1541],		/* Broadcast packet */
 		*pptr;			/* Pointer into packet */
   struct sockaddr_in srcaddr;		/* Source address */
   char		srcname[1024];		/* Source hostname */
   unsigned	address;		/* Source address (host order) */
   struct hostent *srchost;		/* Host entry for source address */
-  cups_ptype_t	type;			/* Printer type */
-  ipp_pstate_t	state;			/* Printer state */
+  unsigned	type;			/* Printer type */
+  unsigned	state;			/* Printer state */
   char		uri[HTTP_MAX_URI],	/* Printer URI */
 		method[HTTP_MAX_URI],	/* Method portion of URI */
 		username[HTTP_MAX_URI],	/* Username portion of URI */
@@ -1089,8 +1153,8 @@ UpdateCUPSBrowse(void)
   */
 
   len = sizeof(srcaddr);
-  if ((bytes = recvfrom(BrowseSocket, packet, sizeof(packet), 0, 
-                        (struct sockaddr *)&srcaddr, &len)) <= 0)
+  if ((bytes = recvfrom(BrowseSocket, packet, sizeof(packet) - 1, 0, 
+                        (struct sockaddr *)&srcaddr, &len)) < 0)
   {
    /*
     * "Connection refused" is returned under Linux if the destination port
@@ -1206,8 +1270,7 @@ UpdateCUPSBrowse(void)
   * Parse packet...
   */
 
-  if (sscanf(packet, "%x%x%1023s", (unsigned *)&type, (unsigned *)&state,
-             uri) < 3)
+  if (sscanf(packet, "%x%x%1023s", &type, &state, uri) < 3)
   {
     LogMessage(L_WARN, "UpdateCUPSBrowse: Garbled browse packet - %s",
                packet);
@@ -1236,7 +1299,7 @@ UpdateCUPSBrowse(void)
     if (*pptr == '\"')
       pptr ++;
 
-    while (*pptr && isspace(*pptr))
+    while (*pptr && isspace(*pptr & 255))
       pptr ++;
 
     if (*pptr == '\"')
@@ -1252,7 +1315,7 @@ UpdateCUPSBrowse(void)
       if (*pptr == '\"')
 	pptr ++;
 
-      while (*pptr && isspace(*pptr))
+      while (*pptr && isspace(*pptr & 255))
 	pptr ++;
 
       if (*pptr == '\"')
@@ -1313,7 +1376,8 @@ UpdateCUPSBrowse(void)
   * Process the browse data...
   */
 
-  ProcessBrowseData(uri, type, state, location, info, make_model);
+  ProcessBrowseData(uri, (cups_ptype_t)type, (ipp_pstate_t)state, location,
+                    info, make_model);
 }
 
 
@@ -1906,5 +1970,5 @@ UpdateSLPBrowse(void)
 
 
 /*
- * End of "$Id: dirsvc.c,v 1.126 2003/10/22 14:41:33 mike Exp $".
+ * End of "$Id: dirsvc.c,v 1.137 2004/09/09 15:10:18 mike Exp $".
  */
