@@ -1,5 +1,5 @@
 /*
- * "$Id: file.c,v 1.5 2003/11/04 16:37:44 mike Exp $"
+ * "$Id: file.c,v 1.9 2004/09/09 15:10:18 mike Exp $"
  *
  *   File functions for the Common UNIX Printing System (CUPS).
  *
@@ -8,7 +8,7 @@
  *   our own file functions allows us to provide transparent support of
  *   gzip'd print files, PPD files, etc.
  *
- *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -20,9 +20,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636-3142 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -311,6 +311,12 @@ cupsFileOpen(const char *filename,	/* I - Name of file */
     return (NULL);
   }
 
+ /*
+  * Don't pass this file to child processes...
+  */
+
+  fcntl(fp->fd, F_SETFD, fcntl(fp->fd, F_GETFD) | FD_CLOEXEC);
+
   if (*mode == 'a')
     fp->pos = lseek(fp->fd, 0, SEEK_END);
   else
@@ -551,7 +557,9 @@ cupsFileSeek(cups_file_t *fp,		/* I - CUPS file */
       fp->pos = pos;
       fp->ptr = NULL;
       fp->end = NULL;
+#ifdef HAVE_LIBZ
       fp->eof = 0;
+#endif /* HAVE_LIBZ */
     }
   }
   else if (pos >= (fp->pos + bytes))
@@ -577,6 +585,9 @@ cupsFileSeek(cups_file_t *fp,		/* I - CUPS file */
       fp->pos = pos;
       fp->ptr = NULL;
       fp->end = NULL;
+#ifdef HAVE_LIBZ
+      fp->eof = 0;
+#endif /* HAVE_LIBZ */
     }
   }
   else
@@ -587,6 +598,9 @@ cupsFileSeek(cups_file_t *fp,		/* I - CUPS file */
     */
 
     fp->ptr = fp->buf + pos - fp->pos;
+#ifdef HAVE_LIBZ
+    fp->eof = 0;
+#endif /* HAVE_LIBZ */
   }
 
   return (pos);
@@ -638,8 +652,10 @@ static int				/* O - Number of bytes or -1 */
 cups_fill(cups_file_t *fp)		/* I - CUPS file */
 {
   int			bytes;		/* Number of bytes read */
+#ifdef HAVE_LIBZ
   const unsigned char	*ptr,		/* Pointer into buffer */
 			*end;		/* End of buffer */
+#endif /* HAVE_LIBZ */
 
 
  /*
@@ -971,5 +987,5 @@ cups_write(int        fd,		/* I - File descriptor */
 
 
 /*
- * End of "$Id: file.c,v 1.5 2003/11/04 16:37:44 mike Exp $".
+ * End of "$Id: file.c,v 1.9 2004/09/09 15:10:18 mike Exp $".
  */
