@@ -1,9 +1,9 @@
 /*
- * "$Id: job.c,v 1.222 2003/11/05 19:11:42 mike Exp $"
+ * "$Id: job.c,v 1.230 2004/09/09 15:10:18 mike Exp $"
  *
  *   Job management routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636-3142 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -106,6 +106,7 @@ AddJob(int        priority,	/* I - Job priority */
 
   job->id       = NextJobId ++;
   job->priority = priority;
+  job->pipe     = -1;
   SetString(&job->dest, dest);
 
   NumJobs ++;
@@ -583,7 +584,8 @@ LoadAllJobs(void)
       * Assign the job ID...
       */
 
-      job->id = atoi(dent->d_name + 1);
+      job->id   = atoi(dent->d_name + 1);
+      job->pipe = -1;
 
       LogMessage(L_DEBUG, "LoadAllJobs: Loading attributes for job %d...\n",
                  job->id);
@@ -940,7 +942,7 @@ SaveJob(int id)			/* I - Job ID */
   }
 
   fchmod(fd, 0600);
-  fchown(fd, getuid(), Group);
+  fchown(fd, RunUser, Group);
 
   ippWriteFile(fd, job->attrs);
 
@@ -1175,7 +1177,8 @@ StartJob(int       id,			/* I - Job ID */
   int		num_filters;		/* Number of filters for job */
   mime_filter_t	*filters;		/* Filters for job */
   char		method[255],		/* Method for output */
-		*optptr;		/* Pointer to options */
+		*optptr,		/* Pointer to options */
+		*valptr;		/* Pointer in value string */
   ipp_attribute_t *attr;		/* Current attribute */
   int		pid;			/* Process ID of new filter process */
   int		banner_page;		/* 1 if banner page, 0 otherwise */
@@ -1582,18 +1585,14 @@ StartJob(int       id,			/* I - Job ID */
 	  case IPP_TAG_KEYWORD :
 	  case IPP_TAG_CHARSET :
 	  case IPP_TAG_LANGUAGE :
-	      if (strchr(attr->values[i].string.text, ' ') != NULL ||
-		  strchr(attr->values[i].string.text, '\t') != NULL ||
-		  strchr(attr->values[i].string.text, '\n') != NULL)
+	      for (valptr = attr->values[i].string.text; *valptr;)
 	      {
-		strlcat(optptr, "\'", optlength - (optptr - options));
-		strlcat(optptr, attr->values[i].string.text,
-		        optlength - (optptr - options));
-		strlcat(optptr, "\'", optlength - (optptr - options));
+	        if (strchr(" \t\n\\\'\"", *valptr))
+		  *optptr++ = '\\';
+		*optptr++ = *valptr++;
 	      }
-	      else
-		strlcat(optptr, attr->values[i].string.text,
-		        optlength - (optptr - options));
+
+	      *optptr = '\0';
 	      break;
 
           default :
@@ -1672,8 +1671,8 @@ StartJob(int       id,			/* I - Job ID */
         snprintf(language, sizeof(language), "LANG=%c%c_%c%c",
 	         attr->values[0].string.text[0],
 		 attr->values[0].string.text[1],
-		 toupper(attr->values[0].string.text[3]),
-		 toupper(attr->values[0].string.text[4]));
+		 toupper(attr->values[0].string.text[3] & 255),
+		 toupper(attr->values[0].string.text[4] & 255));
         break;
   }
 
@@ -1830,7 +1829,7 @@ StartJob(int       id,			/* I - Job ID */
   * Now create processes for all of the filters...
   */
 
-  if (pipe(statusfds))
+  if (cupsdPipe(statusfds))
   {
     LogMessage(L_ERROR, "Unable to create job status pipes - %s.",
 	       strerror(errno));
@@ -1838,6 +1837,10 @@ StartJob(int       id,			/* I - Job ID */
              "Unable to create status pipes - %s.", strerror(errno));
 
     AddPrinterHistory(printer);
+
+    if (filters != NULL)
+      free(filters);
+
     return;
   }
 
@@ -1851,6 +1854,23 @@ StartJob(int       id,			/* I - Job ID */
   filterfds[1][0] = open("/dev/null", O_RDONLY);
   filterfds[1][1] = -1;
 
+  if (filterfds[1][0] < 0)
+  {
+    LogMessage(L_ERROR, "Unable to open \"/dev/null\" - %s.", strerror(errno));
+    snprintf(printer->state_message, sizeof(printer->state_message),
+             "Unable to open \"/dev/null\" - %s.", strerror(errno));
+
+    AddPrinterHistory(printer);
+
+    if (filters != NULL)
+      free(filters);
+
+    CancelJob(current->id, 0);
+    return;
+  }
+
+  fcntl(filterfds[1][0], F_SETFD, fcntl(filterfds[1][0], F_GETFD) | FD_CLOEXEC);
+
   LogMessage(L_DEBUG, "StartJob: filterfds[%d] = [ %d %d ]", 1, filterfds[1][0],
              filterfds[1][1]);
 
@@ -1874,7 +1894,7 @@ StartJob(int       id,			/* I - Job ID */
 
     if (i < (num_filters - 1) ||
 	strncmp(printer->device_uri, "file:", 5) != 0)
-      pipe(filterfds[slot]);
+      cupsdPipe(filterfds[slot]);
     else
     {
       filterfds[slot][0] = -1;
@@ -1884,6 +1904,26 @@ StartJob(int       id,			/* I - Job ID */
       else
 	filterfds[slot][1] = open(printer->device_uri + 5,
 	                          O_WRONLY | O_CREAT | O_TRUNC, 0600);
+
+      if (filterfds[slot][1] < 0)
+      {
+        LogMessage(L_ERROR, "Unable to open output file \"%s\" - %s.",
+	           printer->device_uri, strerror(errno));
+        snprintf(printer->state_message, sizeof(printer->state_message),
+		 "Unable to open output file \"%s\" - %s.",
+	         printer->device_uri, strerror(errno));
+
+	AddPrinterHistory(printer);
+
+	if (filters != NULL)
+	  free(filters);
+
+	CancelJob(current->id, 0);
+	return;
+      }
+
+      fcntl(filterfds[slot][1], F_SETFD,
+            fcntl(filterfds[slot][1], F_GETFD) | FD_CLOEXEC);
     }
 
     LogMessage(L_DEBUG, "StartJob: filter = \"%s\"", command);
@@ -1894,8 +1934,10 @@ StartJob(int       id,			/* I - Job ID */
                         filterfds[slot][1], statusfds[1], 0,
 			current->procs + i);
 
-    close(filterfds[!slot][0]);
-    close(filterfds[!slot][1]);
+    if (filterfds[!slot][0] >= 0)
+      close(filterfds[!slot][0]);
+    if (filterfds[!slot][1] >= 0)
+      close(filterfds[!slot][1]);
 
     if (pid == 0)
     {
@@ -1948,6 +1990,24 @@ StartJob(int       id,			/* I - Job ID */
     filterfds[slot][0] = -1;
     filterfds[slot][1] = open("/dev/null", O_WRONLY);
 
+    if (filterfds[slot][1] < 0)
+    {
+      LogMessage(L_ERROR, "Unable to open \"/dev/null\" - %s.", strerror(errno));
+      snprintf(printer->state_message, sizeof(printer->state_message),
+               "Unable to open \"/dev/null\" - %s.", strerror(errno));
+
+      AddPrinterHistory(printer);
+
+      if (filters != NULL)
+	free(filters);
+
+      CancelJob(current->id, 0);
+      return;
+    }
+
+    fcntl(filterfds[slot][1], F_SETFD,
+          fcntl(filterfds[slot][1], F_GETFD) | FD_CLOEXEC);
+
     LogMessage(L_DEBUG, "StartJob: backend = \"%s\"", command);
     LogMessage(L_DEBUG, "StartJob: filterfds[%d] = [ %d %d ]",
                slot, filterfds[slot][0], filterfds[slot][1]);
@@ -1956,8 +2016,10 @@ StartJob(int       id,			/* I - Job ID */
 			filterfds[slot][1], statusfds[1], 1,
 			current->procs + i);
 
-    close(filterfds[!slot][0]);
-    close(filterfds[!slot][1]);
+    if (filterfds[!slot][0] >= 0)
+      close(filterfds[!slot][0]);
+    if (filterfds[!slot][1] >= 0)
+      close(filterfds[!slot][1]);
 
     if (pid == 0)
     {
@@ -1982,12 +2044,16 @@ StartJob(int       id,			/* I - Job ID */
     filterfds[slot][0] = -1;
     filterfds[slot][1] = -1;
 
-    close(filterfds[!slot][0]);
-    close(filterfds[!slot][1]);
+    if (filterfds[!slot][0] >= 0)
+      close(filterfds[!slot][0]);
+    if (filterfds[!slot][1] >= 0)
+      close(filterfds[!slot][1]);
   }
 
-  close(filterfds[slot][0]);
-  close(filterfds[slot][1]);
+  if (filterfds[slot][0] >= 0)
+    close(filterfds[slot][0]);
+  if (filterfds[slot][1] >= 0)
+    close(filterfds[slot][1]);
 
   close(statusfds[1]);
 
@@ -2065,7 +2131,7 @@ StopJob(int id,			/* I - Job ID */
 	    current->procs[i] = 0;
 	  }
 
-        if (current->pipe)
+        if (current->pipe >= 0)
         {
 	 /*
 	  * Close the pipe and clear the input bit.
@@ -2076,7 +2142,7 @@ StopJob(int id,			/* I - Job ID */
 
           close(current->pipe);
 	  FD_CLR(current->pipe, InputSet);
-	  current->pipe = 0;
+	  current->pipe = -1;
         }
 
         if (current->buffer)
@@ -2104,6 +2170,7 @@ StopJob(int id,			/* I - Job ID */
 void
 UpdateJob(job_t *job)		/* I - Job to check */
 {
+  int		i;		/* Looping var */
   int		bytes;		/* Number of bytes read */
   int		copies;		/* Number of copies printed */
   char		*lineptr,	/* Pointer to end of line in buffer */
@@ -2211,7 +2278,7 @@ UpdateJob(job_t *job)		/* I - Job to check */
     * Skip leading whitespace in the message...
     */
 
-    while (isspace(*message))
+    while (isspace(*message & 255))
       message ++;
 
    /*
@@ -2283,10 +2350,23 @@ UpdateJob(job_t *job)		/* I - Job to check */
 
   if (bytes <= 0)
   {
+   /*
+    * See if all of the filters and the backend have returned their
+    * exit statuses.
+    */
+
+    for (i = 0; job->procs[i]; i ++)
+      if (job->procs[i] > 0)
+	return;
+
+   /*
+    * Handle the end of job stuff...
+    */
+
     LogMessage(L_DEBUG, "UpdateJob: job %d, file %d is complete.",
                job->id, job->current_file - 1);
 
-    if (job->pipe)
+    if (job->pipe >= 0)
     {
      /*
       * Close the pipe and clear the input bit.
@@ -2297,7 +2377,7 @@ UpdateJob(job_t *job)		/* I - Job to check */
 
       close(job->pipe);
       FD_CLR(job->pipe, InputSet);
-      job->pipe = 0;
+      job->pipe = -1;
     }
 
     if (job->status < 0)
@@ -2558,7 +2638,6 @@ start_process(const char *command,	/* I - Full path to command */
 	      int        root,		/* I - Run as root? */
               int        *pid)		/* O - Process ID */
 {
-  int	fd;				/* Looping var */
 #if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
   struct sigaction	action;		/* POSIX signal handler */
 #endif /* HAVE_SIGACTION && !HAVE_SIGSET */
@@ -2591,13 +2670,6 @@ start_process(const char *command,	/* I - Full path to command */
       dup(errfd);
     }
 
-   /*
-    * Close extra file descriptors...
-    */
-
-    for (fd = 3; fd < MaxFDs; fd ++)
-      close(fd);
-
    /*
     * Change the priority of the process based on the FilterNice setting.
     * (this is not done for backends...)
@@ -2610,7 +2682,7 @@ start_process(const char *command,	/* I - Full path to command */
     * Change user to something "safe"...
     */
 
-    if (!root && getuid() == 0)
+    if (!root && !RunUser)
     {
      /*
       * Running as root, so change to non-priviledged user...
@@ -2739,5 +2811,5 @@ set_hold_until(job_t *job, 		/* I - Job to update */
 
 
 /*
- * End of "$Id: job.c,v 1.222 2003/11/05 19:11:42 mike Exp $".
+ * End of "$Id: job.c,v 1.230 2004/09/09 15:10:18 mike Exp $".
  */
