@@ -1,10 +1,10 @@
 /*
- * "$Id: listen.c,v 1.19 2003/07/20 02:51:54 mike Exp $"
+ * "$Id: listen.c,v 1.24 2004/06/29 03:27:35 mike Exp $"
  *
  *   Server listening routines for the Common UNIX Printing System (CUPS)
  *   scheduler.
  *
- *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -16,9 +16,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636-3142 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -141,7 +141,7 @@ StartListening(void)
   * Setup socket listeners...
   */
 
-  for (i = NumListeners, lis = Listeners; i > 0; i --, lis ++)
+  for (i = NumListeners, lis = Listeners, LocalPort = 0; i > 0; i --, lis ++)
   {
     LogMessage(L_DEBUG, "StartListening: address=%08x port=%d",
                (unsigned)ntohl(lis->address.sin_addr.s_addr),
@@ -152,9 +152,13 @@ StartListening(void)
     * "any" address...
     */
 
-    if (ntohl(lis->address.sin_addr.s_addr) == 0x7f000001 ||
-        ntohl(lis->address.sin_addr.s_addr) == 0x00000000)
-      LocalPort = ntohs(lis->address.sin_port);
+    if (!LocalPort &&
+        (ntohl(lis->address.sin_addr.s_addr) == 0x7f000001 ||
+         ntohl(lis->address.sin_addr.s_addr) == 0x00000000))
+    {
+      LocalPort       = ntohs(lis->address.sin_port);
+      LocalEncryption = lis->encryption;
+    }
 
    /*
     * Create a socket for listening...
@@ -162,8 +166,9 @@ StartListening(void)
 
     if ((lis->fd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
     {
-      LogMessage(L_ERROR, "StartListening: Unable to open listen socket - %s.",
-                 strerror(errno));
+      LogMessage(L_ERROR, "StartListening: Unable to open listen socket for address %08x:%d - %s.",
+                 (unsigned)ntohl(lis->address.sin_addr.s_addr),
+		 ntohs(lis->address.sin_port), strerror(errno));
       exit(errno);
     }
 
@@ -186,7 +191,9 @@ StartListening(void)
 
     if (bind(lis->fd, (struct sockaddr *)&(lis->address), sizeof(lis->address)) < 0)
     {
-      LogMessage(L_ERROR, "StartListening: Unable to bind socket - %s.", strerror(errno));
+      LogMessage(L_ERROR, "StartListening: Unable to bind socket for address %08x:%d - %s.",
+                 (unsigned)ntohl(lis->address.sin_addr.s_addr),
+		 ntohs(lis->address.sin_port), strerror(errno));
       exit(errno);
     }
 
@@ -196,12 +203,28 @@ StartListening(void)
 
     if (listen(lis->fd, ListenBackLog) < 0)
     {
-      LogMessage(L_ERROR, "StartListening: Unable to listen for clients - %s.",
-                 strerror(errno));
+      LogMessage(L_ERROR, "StartListening: Unable to listen for clients on address %08x:%d - %s.",
+                 (unsigned)ntohl(lis->address.sin_addr.s_addr),
+		 ntohs(lis->address.sin_port), strerror(errno));
       exit(errno);
     }
   }
 
+ /*
+  * Make sure that we are listening on localhost!
+  */
+
+  if (!LocalPort)
+  {
+    LogMessage(L_EMERG, "No Listen or Port lines were found to allow access via localhost!");
+
+   /*
+    * Commit suicide...
+    */
+
+    kill(getpid(), SIGTERM);
+  }
+
   ResumeListening();
 }
 
@@ -231,5 +254,5 @@ StopListening(void)
 
 
 /*
- * End of "$Id: listen.c,v 1.19 2003/07/20 02:51:54 mike Exp $".
+ * End of "$Id: listen.c,v 1.24 2004/06/29 03:27:35 mike Exp $".
  */
