@@ -1,9 +1,9 @@
 /*
- * "$Id: log.c,v 1.38 2003/10/09 19:13:30 mike Exp $"
+ * "$Id: log.c,v 1.42 2004/08/23 18:36:50 mike Exp $"
  *
  *   Log file routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -120,7 +120,6 @@ LogMessage(int        level,		/* I - Log level */
 	   ...)				/* I - Additional args as needed */
 {
   int		len;			/* Length of message */
-  char		*ptr;			/* Pointer to message */
   va_list	ap;			/* Argument pointer */
   static const char levels[] =		/* Log levels... */
 		{
@@ -249,8 +248,6 @@ LogMessage(int        level,		/* I - Log level */
     va_end(ap);
   }
 
-  ptr = line;
-
   if (len >= linesize)
     len = linesize - 1;
 
@@ -490,7 +487,7 @@ check_log_file(cups_file_t **log,	/* IO - Log file */
 
     if (strncmp(filename, "/dev/", 5))
     {
-      fchown(cupsFileNumber(*log), getuid(), Group);
+      fchown(cupsFileNumber(*log), RunUser, Group);
       fchmod(cupsFileNumber(*log), LogFilePerm);
     }
   }
@@ -518,7 +515,7 @@ check_log_file(cups_file_t **log,	/* IO - Log file */
 
     if (strncmp(filename, "/dev/", 5))
     {
-      fchown(cupsFileNumber(*log), getuid(), Group);
+      fchown(cupsFileNumber(*log), RunUser, Group);
       fchmod(cupsFileNumber(*log), LogFilePerm);
     }
   }
@@ -528,5 +525,5 @@ check_log_file(cups_file_t **log,	/* IO - Log file */
 
 
 /*
- * End of "$Id: log.c,v 1.38 2003/10/09 19:13:30 mike Exp $".
+ * End of "$Id: log.c,v 1.42 2004/08/23 18:36:50 mike Exp $".
  */
