@@ -1,9 +1,9 @@
 /*
- * "$Id: main.c,v 1.112 2003/11/19 16:52:14 mike Exp $"
+ * "$Id: main.c,v 1.126 2004/09/09 15:53:00 mike Exp $"
  *
  *   Scheduler main loop for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,15 +15,16 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
  * Contents:
  *
  *   main()               - Main entry for the CUPS scheduler.
+ *   cupsdPipe()          - Create a pipe which is closed on exec.
  *   CatchChildSignals()  - Catch SIGCHLD signals...
  *   HoldSignals()        - Hold child and termination signals.
  *   IgnoreChildSignals() - Ignore SIGCHLD signals...
@@ -35,6 +36,7 @@
  *   sigchld_handler()    - Handle 'child' signals from old processes.
  *   sighup_handler()     - Handle 'hangup' signals to reconfigure the scheduler.
  *   sigterm_handler()    - Handle 'terminate' signals that stop the scheduler.
+ *   select_timeout()     - Calculate the select timeout value.
  *   usage()              - Show scheduler usage.
  */
 
@@ -62,6 +64,7 @@ static void	process_children(void);
 static void	sigchld_handler(int sig);
 static void	sighup_handler(int sig);
 static void	sigterm_handler(int sig);
+static long	select_timeout(int fds);
 static void	usage(void);
 
 
@@ -89,6 +92,7 @@ main(int  argc,				/* I - Number of command-line arguments */
   int			i;		/* Looping var */
   char			*opt;		/* Option character */
   int			fg;		/* Run in the foreground */
+  int			fds;		/* Number of ready descriptors select returns */
   fd_set		*input,		/* Input set for select() */
 			*output;	/* Output set for select() */
   client_t		*con;		/* Current client */
@@ -222,14 +226,18 @@ main(int  argc,				/* I - Number of command-line arguments */
       if (wait(&i) < 0)
       {
         perror("cupsd");
-	i = 1;
+	return (1);
+      }
+      else if (WIFEXITED(i))
+      {
+        fprintf(stderr, "cupsd: Child exited with status %d!\n", WEXITSTATUS(i));
+	return (2);
       }
-      else if (i >= 256)
-        fprintf(stderr, "cupsd: Child exited with status %d!\n", i / 256);
       else
-        fprintf(stderr, "cupsd: Child exited on signal %d!\n", i);
-
-      return (i);
+      {
+        fprintf(stderr, "cupsd: Child exited on signal %d!\n", WTERMSIG(i));
+	return (3);
+      }
     }
   }
 
@@ -296,7 +304,10 @@ main(int  argc,				/* I - Number of command-line arguments */
   * Allocate memory for the input and output sets...
   */
 
-  SetSize   = (MaxFDs + 31) / 8;
+  SetSize = (MaxFDs + 31) / 8 + 4;
+  if (SetSize < sizeof(fd_set))
+    SetSize = sizeof(fd_set);
+
   InputSet  = (fd_set *)calloc(1, SetSize);
   OutputSet = (fd_set *)calloc(1, SetSize);
   input     = (fd_set *)calloc(1, SetSize);
@@ -421,15 +432,20 @@ main(int  argc,				/* I - Number of command-line arguments */
   * Loop forever...
   */
 
-  browse_time  = time(NULL);
-  senddoc_time = time(NULL);
-
 #ifdef HAVE_MALLINFO
   mallinfo_time = 0;
 #endif /* HAVE_MALLINFO */
+  browse_time   = time(NULL);
+  senddoc_time  = time(NULL);
+  fds           = 1;
 
   while (!stop_scheduler)
   {
+#ifdef DEBUG
+    LogMessage(L_DEBUG2, "main: Top of loop, dead_children=%d, NeedReload=%d",
+               dead_children, NeedReload);
+#endif /* DEBUG */
+
    /*
     * Check if there are dead children to handle...
     */
@@ -474,10 +490,11 @@ main(int  argc,				/* I - Number of command-line arguments */
 
      /*
       * Restart if all clients are closed and all jobs finished, or
-      * if 60 seconds has elapsed...
+      * if the reload timeout has elapsed...
       */
 
-      if ((NumClients == 0 && !job) || (time(NULL) - ReloadTime) >= 60)
+      if ((NumClients == 0 && (!job || NeedReload != RELOAD_ALL)) ||
+          (time(NULL) - ReloadTime) >= ReloadTimeout)
       {
         if (!ReadConfiguration())
         {
@@ -499,27 +516,10 @@ main(int  argc,				/* I - Number of command-line arguments */
     memcpy(input, InputSet, SetSize);
     memcpy(output, OutputSet, SetSize);
 
-    for (i = NumClients, con = Clients; i > 0; i --, con ++)
-      if (con->http.used > 0)
-        break;
+    timeout.tv_sec  = select_timeout(fds);
+    timeout.tv_usec = 0;
 
-    if (i)
-    {
-      timeout.tv_sec  = 0;
-      timeout.tv_usec = 0;
-    }
-    else
-    {
-     /*
-      * If we have no pending data from a client, see when we really
-      * need to wake up...
-      */
-
-      timeout.tv_sec  = 1;
-      timeout.tv_usec = 0;
-    }
-
-    if ((i = select(MaxFDs, input, output, NULL, &timeout)) < 0)
+    if ((fds = select(MaxFDs, input, output, NULL, &timeout)) < 0)
     {
       char	s[16384],	/* String buffer */
 		*sptr;		/* Pointer into buffer */
@@ -603,8 +603,29 @@ main(int  argc,				/* I - Number of command-line arguments */
       * Write data as needed...
       */
 
+      if (con->pipe_pid && FD_ISSET(con->file, input))
+      {
+       /*
+        * Keep track of pending input from the file/pipe separately
+	* so that we don't needlessly spin on select() when the web
+	* client is not ready to receive data...
+	*/
+
+        con->file_ready = 1;
+
+#ifdef DEBUG
+        LogMessage(L_DEBUG2, "main: Data ready file %d!", con->file);
+#endif /* DEBUG */
+
+	if (!FD_ISSET(con->http.fd, output))
+	{
+	  LogMessage(L_DEBUG2, "main: Removing fd %d from InputSet...", con->file);
+	  FD_CLR(con->file, InputSet);
+	}
+      }
+
       if (FD_ISSET(con->http.fd, output) &&
-          (!con->pipe_pid || FD_ISSET(con->file, input)))
+          (!con->pipe_pid || con->file_ready))
         if (!WriteClient(con))
 	{
 	  con --;
@@ -635,7 +656,7 @@ main(int  argc,				/* I - Number of command-line arguments */
     {
       next = job->next;
 
-      if (job->pipe && FD_ISSET(job->pipe, input))
+      if (job->pipe >= 0 && FD_ISSET(job->pipe, input))
       {
        /*
         * Clear the input bit to avoid updating the next job
@@ -755,10 +776,59 @@ main(int  argc,				/* I - Number of command-line arguments */
       unlink("/var/spool/lp/SCHEDLOCK");
 #endif /* __sgi */
 
+ /*
+  * Free memory used by FD sets and return...
+  */
+
+  free(InputSet);
+  free(OutputSet);
+  free(input);
+  free(output);
+
   return (!stop_scheduler);
 }
 
 
+/*
+ * 'cupsdPipe()' - Create a pipe which is closed on exec.
+ */
+
+int					/* O - 0 on success, -1 on error */
+cupsdPipe(int *fds)			/* O - Pipe file descriptors (2) */
+{
+ /*
+  * Create the pipe...
+  */
+
+  if (pipe(fds))
+    return (-1);
+
+ /*
+  * Set the "close on exec" flag on each end of the pipe...
+  */
+
+  if (fcntl(fds[0], F_SETFD, fcntl(fds[0], F_GETFD) | FD_CLOEXEC))
+  {
+    close(fds[0]);
+    close(fds[1]);
+    return (-1);
+  }
+
+  if (fcntl(fds[1], F_SETFD, fcntl(fds[1], F_GETFD) | FD_CLOEXEC))
+  {
+    close(fds[0]);
+    close(fds[1]);
+    return (-1);
+  }
+
+ /*
+  * Return 0 indicating success...
+  */
+
+  return (0);
+}
+
+
 /*
  * 'CatchChildSignals()' - Catch SIGCHLD signals...
  */
@@ -993,11 +1063,12 @@ process_children(void)
 
     if (status)
     {
-      if (status < 256)
-	LogMessage(L_ERROR, "PID %d crashed on signal %d!", pid, status);
+      if (WIFSTOPPED(status))
+	LogMessage(L_ERROR, "PID %d crashed on signal %d!", pid,
+	           WSTOPSIG(status));
       else
 	LogMessage(L_ERROR, "PID %d stopped with status %d!", pid,
-	           status / 256);
+	           WEXITSTATUS(status));
 
       if (LogLevel < L_DEBUG)
         LogMessage(L_INFO, "Hint: Try setting the LogLevel to \"debug\" to find out more.");
@@ -1114,6 +1185,161 @@ sigterm_handler(int sig)		/* I - Signal */
 }
 
 
+/*
+ * 'select_timeout()' - Calculate the select timeout value.
+ *
+ */
+
+static long				/* O - Number of seconds */
+select_timeout(int fds)			/* I - Number of ready descriptors select returned */
+{
+  int			i;		/* Looping var */
+  long			timeout;	/* Timeout for select */
+  time_t		now;		/* Current time */
+  client_t		*con;		/* Client information */
+  printer_t		*p;		/* Printer information */
+  job_t			*job;		/* Job information */
+  const char		*why;		/* Debugging aid */
+
+
+ /*
+  * Check to see if any of the clients have pending data to be
+  * processed; if so, the timeout should be 0...
+  */
+
+  for (i = NumClients, con = Clients; i > 0; i --, con ++)
+    if (con->http.used > 0)
+      return (0);
+
+ /*
+  * If select has been active in the last second (fds != 0) or we have
+  * many resources in use then don't bother trying to optimize the
+  * timeout, just make it 1 second.
+  */
+
+  if (fds || NumClients > 50)
+    return (1);
+
+ /*
+  * Otherwise, check all of the possible events that we need to wake for...
+  */
+
+  now     = time(NULL);
+  timeout = now + 86400;		/* 86400 == 1 day */
+  why     = "do nothing";
+
+ /*
+  * Check the activity and close old clients...
+  */
+
+  for (i = NumClients, con = Clients; i > 0; i --, con ++)
+    if ((con->http.activity + Timeout) < timeout)
+    {
+      timeout = con->http.activity + Timeout;
+      why     = "timeout a client connection";
+    }
+
+ /*
+  * Update the browse list as needed...
+  */
+
+  if (Browsing && BrowseProtocols)
+  {
+#ifdef HAVE_LIBSLP
+    if ((BrowseProtocols & BROWSE_SLP) && (BrowseSLPRefresh < timeout))
+    {
+      timeout = BrowseSLPRefresh;
+      why     = "update SLP browsing";
+    }
+#endif /* HAVE_LIBSLP */
+
+    if (BrowseProtocols & BROWSE_CUPS)
+    {
+      for (p = Printers; p != NULL; p = p->next)
+      {
+	if (p->type & CUPS_PRINTER_REMOTE)
+	{
+	  if ((p->browse_time + BrowseTimeout) < timeout)
+	  {
+	    timeout = p->browse_time + BrowseTimeout;
+	    why     = "browse timeout a printer";
+	  }
+	}
+	else if (!(p->type & CUPS_PRINTER_IMPLICIT))
+	{
+	  if (BrowseInterval && (p->browse_time + BrowseInterval) < timeout)
+	  {
+	    timeout = p->browse_time + BrowseInterval;
+	    why     = "send browse update";
+	  }
+	}
+      }
+    }
+  }
+
+ /*
+  * Check for any active jobs...
+  */
+
+  if (timeout > (now + 10))
+  {
+    for (job = Jobs; job != NULL; job = job->next)
+      if (job->state->values[0].integer <= IPP_JOB_PROCESSING)
+      {
+	timeout = now + 10;
+	why     = "process active jobs";
+	break;
+      }
+  }
+
+#ifdef HAVE_MALLINFO
+ /*
+  * Log memory usage every minute...
+  */
+
+  if (LogLevel >= L_DEBUG && (mallinfo_time + 60) < timeout)
+  {
+    timeout = mallinfo_time + 60;
+    why     = "display memory usage";
+  }
+#endif /* HAVE_MALLINFO */
+
+ /*
+  * Update the root certificate when needed...
+  */
+
+  if (RootCertDuration && (RootCertTime + RootCertDuration) < timeout)
+  {
+    timeout = RootCertTime + RootCertDuration;
+    why     = "update root certificate";
+  }
+
+ /*
+  * Adjust from absolute to relative time.  If p->browse_time above
+  * was 0 then we can end up with a negative value here, so check.
+  * We add 1 second to the timeout since events occur after the
+  * timeout expires, and limit the timeout to 86400 seconds (1 day)
+  * to avoid select() timeout limits present on some operating
+  * systems...
+  */
+
+  timeout = timeout - now + 1;
+
+  if (timeout < 1)
+    timeout = 1;
+  else if (timeout > 86400)
+    timeout = 86400;
+
+ /*
+  * Log and return the timeout value...
+  */
+
+  LogMessage(L_DEBUG2, "select_timeout: %ld seconds to %s", timeout, why);
+
+  return (timeout);
+}
+
+
 /*
  * 'usage()' - Show scheduler usage.
  */
@@ -1127,5 +1353,5 @@ usage(void)
 
 
 /*
- * End of "$Id: main.c,v 1.112 2003/11/19 16:52:14 mike Exp $".
+ * End of "$Id: main.c,v 1.126 2004/09/09 15:53:00 mike Exp $".
  */
