@@ -1,10 +1,10 @@
 /*
- * "$Id: network.c,v 1.11 2002/12/17 19:00:17 swdev Exp $"
+ * "$Id: network.c,v 1.14 2004/08/18 16:02:44 mike Exp $"
  *
  *   Network interface functions for the Common UNIX Printing System
  *   (CUPS) scheduler.
  *
- *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -16,9 +16,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636-3142 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  */
@@ -129,6 +129,8 @@ NetIFFree(void)
 void
 NetIFUpdate(void)
 {
+  int		i;		/* Looping var */
+  listener_t	*lis;		/* Listen address */
   cups_netif_t	*temp;		/* Current interface */
   struct ifaddrs *addrs,	/* Interface address list */
 		*addr;		/* Current interface address */
@@ -193,6 +195,19 @@ NetIFUpdate(void)
         ntohl(temp->address.sin_addr.s_addr) != 0x7f000001)
       temp->is_local = 1;
 
+   /*
+    * Determine which port to use when advertising printers...
+    */
+
+    for (i = NumListeners, lis = Listeners; i > 0; i --, lis ++)
+      if (lis->address.sin_addr.s_addr == 0x00000000 ||
+          (lis->address.sin_addr.s_addr & temp->mask.sin_addr.s_addr) ==
+	      temp->address.sin_addr.s_addr)
+      {
+        temp->port = ntohs(lis->address.sin_port);
+	break;
+      }
+
    /*
     * Finally, try looking up the hostname for the address as needed...
     */
@@ -472,5 +487,5 @@ freeifaddrs(struct ifaddrs *addrs)	/* I - Interface list to free */
 
 
 /*
- * End of "$Id: network.c,v 1.11 2002/12/17 19:00:17 swdev Exp $".
+ * End of "$Id: network.c,v 1.14 2004/08/18 16:02:44 mike Exp $".
  */
