@@ -1,10 +1,10 @@
 /*
- * "$Id: network.h,v 1.2 2002/12/17 19:00:17 swdev Exp $"
+ * "$Id: network.h,v 1.5 2004/08/23 18:00:59 mike Exp $"
  *
  *   Network interface definitions for the Common UNIX Printing System
  *   (CUPS) scheduler.
  *
- *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -16,9 +16,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  */
@@ -33,7 +33,8 @@ typedef struct cups_netif_str		/**** Network interface data ****/
   char			name[32],	/* Network interface name */
 			hostname[HTTP_MAX_HOST];
 					/* Hostname associated with interface */
-  int			is_local;	/* Local (not point-to-point) interface? */
+  int			is_local,	/* Local (not point-to-point) interface? */
+			port;		/* Listen port */
   struct sockaddr_in	address,	/* Network address */
 			mask,		/* Network mask */
 			broadcast;	/* Broadcast address */
@@ -59,5 +60,5 @@ extern void		NetIFUpdate(void);
 
 
 /*
- * End of "$Id: network.h,v 1.2 2002/12/17 19:00:17 swdev Exp $".
+ * End of "$Id: network.h,v 1.5 2004/08/23 18:00:59 mike Exp $".
  */
