@@ -1,9 +1,9 @@
 /*
- * "$Id: printers.h,v 1.35 2003/04/26 17:09:56 mike Exp $"
+ * "$Id: printers.h,v 1.39 2004/08/23 18:00:59 mike Exp $"
  *
  *   Printer definitions for the Common UNIX Printing System (CUPS) scheduler.
  *
- *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  */
@@ -83,10 +83,15 @@ typedef struct printer_str
  * Globals...
  */
 
-VAR ipp_t		*CommonData VALUE(NULL);/* Common printer object attrs */
-VAR printer_t		*Printers VALUE(NULL);	/* Printer list */
+VAR ipp_t		*CommonData	VALUE(NULL);
+					/* Common printer object attrs */
+VAR int			NumPrinters	VALUE(0);
+					/* Number of printers */
+VAR printer_t		*Printers	VALUE(NULL);
+					/* Printer list */
 VAR printer_t		*DefaultPrinter VALUE(NULL);
-						/* Default printer */
+					/* Default printer */
+
 
 /*
  * Prototypes...
@@ -97,6 +102,7 @@ extern void		AddPrinterFilter(printer_t *p, const char *filter);
 extern void		AddPrinterHistory(printer_t *p);
 extern void		AddPrinterUser(printer_t *p, const char *username);
 extern quota_t		*AddQuota(printer_t *p, const char *username);
+extern void		CreateCommonData(void);
 extern void		DeleteAllPrinters(void);
 extern void		DeletePrinter(printer_t *p, int update);
 extern void		DeletePrinterFilters(printer_t *p);
@@ -122,5 +128,5 @@ extern void		WritePrintcap(void);
 
 
 /*
- * End of "$Id: printers.h,v 1.35 2003/04/26 17:09:56 mike Exp $".
+ * End of "$Id: printers.h,v 1.39 2004/08/23 18:00:59 mike Exp $".
  */
