@@ -1,9 +1,9 @@
 /*
- * "$Id: server.c,v 1.14 2003/04/10 20:14:06 mike Exp $"
+ * "$Id: server.c,v 1.17 2004/09/09 15:10:18 mike Exp $"
  *
  *   Server start/stop routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636-3142 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -93,10 +93,13 @@ StartServer(void)
   * Create a pipe for CGI processes...
   */
 
-  pipe(CGIPipes);
-
-  LogMessage(L_DEBUG2, "StartServer: Adding fd %d to InputSet...", CGIPipes[0]);
-  FD_SET(CGIPipes[0], InputSet);
+  if (cupsdPipe(CGIPipes))
+    LogMessage(L_ERROR, "StartServer: Unable to create pipes for CGI status!");
+  else
+  {
+    LogMessage(L_DEBUG2, "StartServer: Adding fd %d to InputSet...", CGIPipes[0]);
+    FD_SET(CGIPipes[0], InputSet);
+  }
 }
 
 
@@ -180,5 +183,5 @@ StopServer(void)
 
 
 /*
- * End of "$Id: server.c,v 1.14 2003/04/10 20:14:06 mike Exp $".
+ * End of "$Id: server.c,v 1.17 2004/09/09 15:10:18 mike Exp $".
  */
