@@ -1,9 +1,9 @@
 /*
- * "$Id: type.c,v 1.22 2003/04/07 18:03:28 mike Exp $"
+ * "$Id: type.c,v 1.26 2004/08/23 18:36:50 mike Exp $"
  *
  *   MIME typing routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2004 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -175,7 +175,7 @@ mimeAddTypeRule(mime_type_t *mt,	/* I - Type to add to */
 
   while (*rule != '\0')
   {
-    while (isspace(*rule))
+    while (isspace(*rule & 255))
       rule ++;
 
     if (*rule == '(')
@@ -285,13 +285,13 @@ mimeAddTypeRule(mime_type_t *mt,	/* I - Type to add to */
       invert = 1;
       rule ++;
     }
-    else if (isalnum(*rule))
+    else if (isalnum(*rule & 255))
     {
      /*
       * Read an extension name or a function...
       */
 
-      for (ptr = name; isalnum(*rule) && (ptr - name) < (sizeof(name) - 1);)
+      for (ptr = name; isalnum(*rule & 255) && (ptr - name) < (sizeof(name) - 1);)
         *ptr++ = *rule++;
 
       *ptr       = '\0';
@@ -313,7 +313,7 @@ mimeAddTypeRule(mime_type_t *mt,	/* I - Type to add to */
 	  while ((ptr - value[num_values]) < (sizeof(value[0]) - 1) &&
 	         *rule != '\0' && *rule != ',' && *rule != ')')
 	  {
-	    if (isspace(*rule))
+	    if (isspace(*rule & 255))
 	    {
 	     /*
 	      * Ignore whitespace...
@@ -346,7 +346,7 @@ mimeAddTypeRule(mime_type_t *mt,	/* I - Type to add to */
 	      while (*rule != '>' && *rule != '\0' &&
 	             (ptr - value[num_values]) < (sizeof(value[0]) - 1))
 	      {
-	        if (isxdigit(rule[0]) && isxdigit(rule[1]))
+	        if (isxdigit(rule[0] & 255) && isxdigit(rule[1] & 255))
 		{
 		  if (isdigit(*rule))
 		    *ptr = (*rule++ - '0') << 4;
@@ -499,15 +499,15 @@ mimeAddTypeRule(mime_type_t *mt,	/* I - Type to add to */
 	    if (length[1] == 1)
 	      temp->value.charv = value[1][0];
 	    else
-	      temp->value.charv = strtol(value[1], NULL, 0);
+	      temp->value.charv = (char)strtol(value[1], NULL, 0);
 	    break;
 	case MIME_MAGIC_SHORT :
 	    temp->offset       = strtol(value[0], NULL, 0);
-	    temp->value.shortv = strtol(value[1], NULL, 0);
+	    temp->value.shortv = (short)strtol(value[1], NULL, 0);
 	    break;
 	case MIME_MAGIC_INT :
 	    temp->offset     = strtol(value[0], NULL, 0);
-	    temp->value.intv = strtol(value[1], NULL, 0);
+	    temp->value.intv = (int)strtol(value[1], NULL, 0);
 	    break;
 	case MIME_MAGIC_LOCALE :
 	    if (length[0] > (sizeof(temp->value.localev) - 1))
@@ -718,7 +718,7 @@ checkrules(const char   *filename,	/* I - Filename */
 	    */
 
             cupsFileSeek(fp, rules->offset);
-	    buflength = cupsFileRead(fp, buffer, sizeof(buffer));
+	    buflength = cupsFileRead(fp, (char *)buffer, sizeof(buffer));
 	    bufoffset = rules->offset;
 	  }
 
@@ -759,7 +759,7 @@ checkrules(const char   *filename,	/* I - Filename */
 	    */
 
             cupsFileSeek(fp, rules->offset);
-	    buflength = cupsFileRead(fp, buffer, sizeof(buffer));
+	    buflength = cupsFileRead(fp, (char *)buffer, sizeof(buffer));
 	    bufoffset = rules->offset;
 	  }
 
@@ -802,7 +802,7 @@ checkrules(const char   *filename,	/* I - Filename */
 	    */
 
             cupsFileSeek(fp, rules->offset);
-	    buflength = cupsFileRead(fp, buffer, sizeof(buffer));
+	    buflength = cupsFileRead(fp, (char *)buffer, sizeof(buffer));
 	    bufoffset = rules->offset;
 	  }
 
@@ -831,7 +831,7 @@ checkrules(const char   *filename,	/* I - Filename */
 	    */
 
             cupsFileSeek(fp, rules->offset);
-	    buflength = cupsFileRead(fp, buffer, sizeof(buffer));
+	    buflength = cupsFileRead(fp, (char *)buffer, sizeof(buffer));
 	    bufoffset = rules->offset;
 	  }
 
@@ -843,7 +843,7 @@ checkrules(const char   *filename,	/* I - Filename */
 	  if ((rules->offset + rules->length) > (bufoffset + buflength))
 	    result = 0;
 	  else
-            result = (strncasecmp(buffer + rules->offset - bufoffset,
+            result = (strncasecmp((char *)buffer + rules->offset - bufoffset,
 	                          rules->value.stringv, rules->length) == 0);
 	  break;
 
@@ -859,7 +859,7 @@ checkrules(const char   *filename,	/* I - Filename */
 	    */
 
             cupsFileSeek(fp, rules->offset);
-	    buflength = cupsFileRead(fp, buffer, sizeof(buffer));
+	    buflength = cupsFileRead(fp, (char *)buffer, sizeof(buffer));
 	    bufoffset = rules->offset;
 	  }
 
@@ -887,7 +887,7 @@ checkrules(const char   *filename,	/* I - Filename */
 	    */
 
             cupsFileSeek(fp, rules->offset);
-	    buflength = cupsFileRead(fp, buffer, sizeof(buffer));
+	    buflength = cupsFileRead(fp, (char *)buffer, sizeof(buffer));
 	    bufoffset = rules->offset;
 	  }
 
@@ -919,7 +919,7 @@ checkrules(const char   *filename,	/* I - Filename */
 	    */
 
             cupsFileSeek(fp, rules->offset);
-	    buflength = cupsFileRead(fp, buffer, sizeof(buffer));
+	    buflength = cupsFileRead(fp, (char *)buffer, sizeof(buffer));
 	    bufoffset = rules->offset;
 	  }
 
@@ -960,7 +960,7 @@ checkrules(const char   *filename,	/* I - Filename */
 	    */
 
             cupsFileSeek(fp, rules->offset);
-	    buflength = cupsFileRead(fp, buffer, sizeof(buffer));
+	    buflength = cupsFileRead(fp, (char *)buffer, sizeof(buffer));
 	    bufoffset = rules->offset;
 	  }
 
@@ -1128,5 +1128,5 @@ patmatch(const char *s,		/* I - String to match against */
 
 
 /*
- * End of "$Id: type.c,v 1.22 2003/04/07 18:03:28 mike Exp $".
+ * End of "$Id: type.c,v 1.26 2004/08/23 18:36:50 mike Exp $".
  */
