@@ -1,9 +1,9 @@
 /*
- * "$Id: lp.c,v 1.45 2003/11/07 20:15:48 mike Exp $"
+ * "$Id: lp.c,v 1.49 2004/09/08 19:01:34 mike Exp $"
  *
  *   "lp" command for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products.
+ *   Copyright 1997-2004 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -520,10 +520,28 @@ main(int  argc,		/* I - Number of command-line arguments */
 
   if (printer == NULL)
   {
-    if (cupsLastError() >= IPP_BAD_REQUEST)
-      fputs("lp: error - scheduler not responding!\n", stderr);
+    val = NULL;
+
+    if ((printer = getenv("LPDEST")) == NULL)
+    {
+      if ((printer = getenv("PRINTER")) != NULL)
+      {
+        if (!strcmp(printer, "lp"))
+          printer = NULL;
+	else
+	  val = "PRINTER";
+      }
+    }
     else
+      val = "LPDEST";
+
+    if (printer && !cupsGetDest(printer, NULL, num_dests, dests))
+      fprintf(stderr, "lp: error - %s environment variable names non-existent destination \"%s\"!\n",
+              val, printer);
+    else if (cupsLastError() == IPP_NOT_FOUND)
       fputs("lp: error - no default destination available.\n", stderr);
+    else
+      fputs("lp: error - scheduler not responding!\n", stderr);
 
     return (1);
   }
@@ -561,7 +579,8 @@ main(int  argc,		/* I - Number of command-line arguments */
 
     if (temp < 0)
     {
-      fputs("lp: unable to create temporary file.\n", stderr);
+      fprintf(stderr, "lp: unable to create temporary file \"%s\" - %s\n",
+              tempfile, strerror(errno));
       return (1);
     }
 
@@ -747,5 +766,5 @@ sighandler(int s)	/* I - Signal number */
 
 
 /*
- * End of "$Id: lp.c,v 1.45 2003/11/07 20:15:48 mike Exp $".
+ * End of "$Id: lp.c,v 1.49 2004/09/08 19:01:34 mike Exp $".
  */
