@@ -1,9 +1,9 @@
 /*
- * "$Id: lpadmin.c,v 1.42 2003/10/06 20:32:55 mike Exp $"
+ * "$Id: lpadmin.c,v 1.44 2004/02/25 20:14:54 mike Exp $"
  *
  *   "lpadmin" command for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2003 by Easy Software Products.
+ *   Copyright 1997-2004 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -1942,7 +1942,7 @@ set_printer_options(http_t        *http,/* I - Server connection */
         strlcpy(keyword, line + 8, sizeof(keyword));
 
 	for (keyptr = keyword; *keyptr; keyptr ++)
-	  if (*keyptr == ':' || isspace(*keyptr))
+	  if (*keyptr == ':' || isspace(*keyptr & 255))
 	    break;
 
         *keyptr = '\0';
@@ -2049,5 +2049,5 @@ validate_name(const char *name)	/* I - Name to check */
 
 
 /*
- * End of "$Id: lpadmin.c,v 1.42 2003/10/06 20:32:55 mike Exp $".
+ * End of "$Id: lpadmin.c,v 1.44 2004/02/25 20:14:54 mike Exp $".
  */
