@@ -1,11 +1,11 @@
 #!/bin/sh
 #
-# "$Id: run-stp-tests.sh,v 1.21 2003/11/19 19:56:42 mike Exp $"
+# "$Id: run-stp-tests.sh,v 1.25 2004/06/28 23:31:34 mike Exp $"
 #
 #   Perform the complete set of IPP compliance tests specified in the
 #   CUPS Software Test Plan.
 #
-#   Copyright 1997-2003 by Easy Software Products, all rights reserved.
+#   Copyright 1997-2004 by Easy Software Products, all rights reserved.
 #
 #   These coded instructions, statements, and computer programs are the
 #   property of Easy Software Products and are protected by Federal
@@ -116,7 +116,7 @@ read usevalgrind
 
 case "$usevalgrind" in
 	Y* | y*)
-		valgrind="valgrind --logfile=/tmp/$user/log/valgrind --error-limit=no --leak-check=yes --trace-children=yes"
+		valgrind="valgrind --tool=memcheck --logfile=/tmp/$user/log/valgrind --error-limit=no --leak-check=yes --trace-children=yes"
 		echo "Using Valgrind; log files can be found in /tmp/$user/log..."
 		;;
 
@@ -181,6 +181,7 @@ ln -s $root/templates /tmp/$user/share
 cat >/tmp/$user/cupsd.conf <<EOF
 Browsing Off
 FileDevice yes
+Printcap
 Listen 127.0.0.1:$port
 User $user
 ServerRoot /tmp/$user
@@ -301,7 +302,8 @@ export HOME
 # Start the server; run as foreground daemon in the background...
 #
 
-echo "Starting scheduler..."
+echo "Starting scheduler:"
+echo "    $valgrind ../scheduler/cupsd -c /tmp/$user/cupsd.conf -f >/tmp/$user/log/debug_log &"
 
 $valgrind ../scheduler/cupsd -c /tmp/$user/cupsd.conf -f >/tmp/$user/log/debug_log &
 cupsd=$!
@@ -454,7 +456,7 @@ else
 fi
 
 if test "x$valgrind" != x; then
-	echo "Valgrind lof files can be found in /tmp/$user/log."
+	echo "Valgrind log files can be found in /tmp/$user/log."
 fi
 
 echo ""
@@ -465,5 +467,5 @@ echo "    $pdffile"
 echo ""
 
 #
-# End of "$Id: run-stp-tests.sh,v 1.21 2003/11/19 19:56:42 mike Exp $"
+# End of "$Id: run-stp-tests.sh,v 1.25 2004/06/28 23:31:34 mike Exp $"
 #
